{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvAlarms.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvAlarms;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, JvTypes, ExtCtrls;

type
  TJvAlarms = class(TComponent)
  private
    FOnAlarm: TOnAlarm;
    FAlarms: TstringList;
    FTimer:TTimer;
    FEnabled: boolean;
    FLast:TTime;
    procedure OnTimer(Sender: TObject);
    procedure SetEnabled(const Value: boolean);
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property Alarms:TstringList read FAlarms write FAlarms;
    property OnAlarm:TOnAlarm read FOnAlarm write FOnAlarm;
    property Active:boolean read FEnabled write SetEnabled default false;
    procedure InsertAlarm(Keyword:string;AlarmTime:TDateTime);
    procedure DeleteAlarm(index:integer);
    function GetAlarmCount:Integer;
    function GetAlarm(index:integer):TAlarm;
  end;

implementation

resourcestring
   RC_Alarm1            =   'Unable to get alarm number :';
   RC_Alarm2            =   '. Index out of bounds';

{*****************************************************}
constructor TJvAlarms.Create(AOwner: TComponent);
begin
   inherited;
   Ftimer:=TTimer.Create(self);
   FTimer.interval:=500;
   FTImer.OnTimer:=OnTimer;
   FTImer.enabled:=false;
   FAlarms:=TstringList.Create;
   FLast:=time;
end;
{*****************************************************}
procedure TJvAlarms.DeleteAlarm(index: integer);
begin
   self.FAlarms.Delete(index);
end;
{*****************************************************}
destructor TJvAlarms.Destroy;
begin
   FTimer.free;
   Falarms.free;
   inherited;
end;
{*****************************************************}
function TJvAlarms.GetAlarm(index: integer): TAlarm;
var
   st:string;
begin
   if index>=Falarms.count then
      raise Exception(RC_Alarm1+IntToStr(index)+RC_Alarm2)
   else
   begin
      st:=Falarms[index];
      result.Keyword:=Copy(st,pos('|',st)+1,length(st));
      result.DateTime:=StrToDateTime(Copy(st,1,pos('|',st)-1));
   end;
end;
{*****************************************************}
function TJvAlarms.GetAlarmCount: Integer;
begin
   result:=self.Falarms.count;
end;
{*****************************************************}
procedure TJvAlarms.InsertAlarm(Keyword: string; AlarmTime: TDateTime);
begin
   FAlarms.Add(DateTimeToStr(AlarmTime)+'|'+KeyWord);
end;
{*****************************************************}
procedure TJvAlarms.OnTimer(Sender: TObject);
var
   i:Integer;
   alarm:TAlarm;
begin
   if FLast<>StrToDateTime(DateTimetoStr(now)) then
   begin
      FLast:=StrToDateTime(DateTimetoStr(now));
      for i:=0 to self.FAlarms.Count-1 do
      begin
         alarm:=GetAlarm(i);
         if (alarm.DateTime=FLast) and (Assigned(FonAlarm)) then
            FOnAlarm(self,alarm.Keyword);
      end;
   end;
end;
{*****************************************************}
procedure TJvAlarms.SetEnabled(const Value: boolean);
begin
   FEnabled := Value;
   FTImer.enabled:=Value;
end;
{*****************************************************}

end.

