{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvAppearingLabel.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvAppearingLabel;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, JvLabel, JvTypes,
  Forms;

type
  TJvAppearThread = class(TThread)
  protected
    procedure Draw;
    procedure Execute; override;
  public
    FDelay:cardinal;
    FOnDraw:TNotifyEvent;
  end;

  TJvAppearingLabel = class(TJvLabel)
  private
    FAutoStart: boolean;
    FInterval: cardinal;
    FPixel: Integer;
    FAppear: TAppear;
    FonAppeared: TNotifyEvent;
    Form:TControl;
    FDestWidth,FDestHeight,FDestLeft,FDestTop:Integer;
    FFirst: cardinal;
    FThread:TJvAppearThread;
    FTag:Integer;
    procedure SetAutoStart(const Value: boolean);
    procedure SetInterval(const Value: cardinal);
    procedure Appearing(Sender: TObject);
    procedure SetFirst(const Value: cardinal);
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property FirstInterval:cardinal read FFirst write SetFirst default 100;
    property Interval:cardinal read FInterval write SetInterval default 20;
    property ScrollPixel:Integer read FPixel write FPixel default 3;
    property AutoStart:boolean read FAutoStart write SetAutoStart default false;
    property AppearFrom:TAppear read FAppear write FAppear default apFromRight;
    property OnAppeared:TNotifyEvent read FonAppeared write FonAppeared;
    procedure Appear;
  end;


implementation

///////////////////////////////////////////////////////////
// TJvAppearingLabel
///////////////////////////////////////////////////////////

{**************************************************}
procedure TJvAppearingLabel.Appear;
begin
   FTag:=0;
   FThread.Resume;
end;
{**************************************************}
procedure TJvAppearingLabel.Appearing(Sender: TObject);
begin
   if FTag=0 then
   begin
      FDestLeft:=self.left;
      FDestTop:=self.top;
      FDestwidth:=self.width;
      FDestHeight:=self.height;
      case Fappear of
           apFromRight:   begin
                             self.Left:=Form.Clientwidth;
                             self.Width:=0;
                          end;
           apFromLeft:self.Left:=-self.width;
           apFromTop:self.top:=-self.height;
           apFromBottom: begin
                            self.top:=form.ClientHeight;
                            self.height:=0;
                         end;
      end;
      FTag:=1;
      FThread.FDelay:=FInterval;
      self.visible:=true;
   end;
   case Fappear of
        apFromRight: begin                            
                        if abs(self.Left-FDestLeft)<Fpixel then
                        begin
                           self.left:=FDestLeft;
                           FThread.Suspend;
                           if Assigned(FonAppeared) then FonAppeared(self);
                        end
                        else
                             self.left:=self.left-FPixel;
                        if self.Width<>FDestwidth then
                        begin
                           if self.left+FDestwidth<Form.ClientWidth then
                              self.width:=FDestwidth
                           else
                              self.width:=form.Clientwidth-self.left-2;
                        end;
                     end;
        apFromLeft:  begin
                        if abs(self.Left-FDestLeft)<Fpixel then
                        begin
                           self.left:=FDestLeft;
                           FThread.Suspend;
                           if Assigned(FonAppeared) then FonAppeared(self);
                        end
                        else
                            self.left:=self.left+FPixel;
                     end;
        apFromTop:   begin
                        if abs(self.top-FDestTop)<Fpixel then
                        begin
                           self.top:=FDestTop;
                           FThread.Suspend;
                           if Assigned(FonAppeared) then FonAppeared(self);
                        end
                        else
                            self.top:=self.top+FPixel;
                     end;
        apFromBottom:begin
                        if abs(self.top-FDestTop)<Fpixel then
                        begin
                           self.top:=FDestTop;
                           FThread.Suspend;
                           if Assigned(FonAppeared) then FonAppeared(self);
                        end
                        else
                            self.top:=self.top-FPixel;
                        if self.height<>FDestHeight then
                        begin
                           if self.top+FDestHeight<Form.Clientheight then
                              self.height:=FDestHeight
                           else
                              self.height:=form.Clientheight-self.top-2;
                        end;
                     end;     
   end;
end;
{**************************************************}
constructor TJvAppearingLabel.Create(AOwner: TComponent);
begin
   inherited;
   FFirst:=100;
   FInterval:=20;
   FPixel:=3;
   FAutoStart:=false;
   FAppear:=apFromRight;
   Form:=AOwner as TControl;
   FTag:=0;

   FThread:=TJvAppearThread.Create(true);
   FThread.FreeOnTerminate:=false;
   FThread.FDelay:=FInterval;
   FThread.FOnDraw:=Appearing;
end;
{**************************************************}
destructor TJvAppearingLabel.Destroy;
begin
   FThread.Terminate;
   while (not FThread.terminated) do
         Application.ProcessMessages;
   FThread.free;
   inherited;
end;
{**************************************************}
procedure TJvAppearingLabel.SetAutoStart(const Value: boolean);
begin
   FAutoStart := Value;
   if Value then visible:=false;
   if (Value) and not(csDesigning in componentstate) then
   begin
      if Value then
         FThread.Resume
      else
         FThread.Suspend;
   end;
end;
{**************************************************}
procedure TJvAppearingLabel.SetFirst(const Value: cardinal);
begin
   FFirst := Value;
   if FTag=0 then FThread.FDelay:=FFirst;
end;
{**************************************************}
procedure TJvAppearingLabel.SetInterval(const Value: cardinal);
begin
   FInterval := Value;
   if FTag<>0 then FThread.FDelay:=Value;
end;
{**************************************************}

///////////////////////////////////////////////////////////
// TJvAppearThread
///////////////////////////////////////////////////////////

{*******************************************************}
procedure TJvAppearThread.Draw;
begin
   if Assigned(FOnDraw) then
     FOnDraw(nil)
end;
{*******************************************************}
procedure TJvAppearThread.Execute;
begin
   while not(Terminated) do
   begin
      SynChronize(Draw);
      sleep(FDelay);
   end;
end;
{*******************************************************}

end.

