{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvAutoSizeCompo.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvAutoSizeCompo;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, StdCtrls;

type
  TJvAutoSizeCompo = class(TComponent)
  private
    FForm: TForm;
    FActive: boolean;
    FResize: TNotifyEvent;
    OldWidth,OldHeight: Integer;
    procedure Resize(Sender: TObject);
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property Active:boolean read FActive write FActive default true;
  end;

implementation

{**************************************************}
constructor TJvAutoSizeCompo.Create(AOwner: TComponent);
begin
  inherited;
  FActive := true;
  FForm := TForm(GetParentForm(TControl(AOwner)));
  Oldwidth := FForm.Width;
  OldHeight := FForm.Height;
  Fresize := FForm.OnResize;
  FForm.OnResize := Resize;
end;
{**************************************************}
destructor TJvAutoSizeCompo.Destroy;
begin
  if FForm<>nil then
    FForm.OnResize:=nil;
  inherited;
end;
{**************************************************}
procedure TJvAutoSizeCompo.Resize(Sender: TObject);
var
  widthRatio, heightRatio: double;
  compIndex: Integer;
begin
  if FActive then
  begin
    if (OldWidth<>0) and (OldHeight<>0) then
    begin
      WidthRatio := FForm.Width / oldWidth;
      HeightRatio := FForm.Height / oldHeight;
      for compIndex := 0 to (FForm.ComponentCount - 1) do
      begin
        if FForm.Components[compIndex] is TControl then
        begin
          with FForm.Components[compIndex] as TControl do
          begin
            if not(FForm.Components[compIndex] is TButton) then
            begin
              Width := round(Width*WidthRatio);
              Height := round(Height*HeightRatio);
            end;
            Left := round(Left*WidthRatio);
            Top := round(top*HeightRatio);
          end;
        end;
      end;
    end;
    OldWidth := FForm.Width;
    OldHeight := FForm.Height;
  end;
  if Assigned(FResize) then
    FResize(Sender);
end;
{**************************************************}
end.
