{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvBrowseFolder.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvBrowseFolder;

{$ObjExportAll On} 

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  ShellApi, Shlobj, Activex, JvBaseDlg, JvFunctions;

type
  TFromDirectory = (fdRootFolder,fdRecycleBin,fdControlPanel,fdDesktop,
    fdDesktopDirectory,fdMyComputer,fdFonts,fdNetHood,fdNetWork,fdPersonal,
    fdPrinters,fdPrograms,fdRecent,fdSendTo,fdStartMenu,fdStartup,fdTemplates);
  TJvFolderPos = (fpDefault, fpScreenCenter, fpFormCenter, fpTopLeft,
    fpTopRight, fpBottomLeft, fpBottomRight);
  TJvDirChange = procedure (Sender: TObject; Directory: string) of object;
  TOptionsDirectory = (odBrowseForComputer, odOnlyDirectory, odOnlyPrinters, odNoBelowDomain,
    odSystemAncestorsOnly, odFileSystemDirectoryOnly, odStatusAvailable, odIncludeFiles,
    odIncludeUrls, odEditBox, odNewDialogStyle, odShareable);
  TOptionsDir = set of TOptionsDirectory;

  TJvBrowseFolder = class(TJvCommonDialog)
  private
    FHandle: Integer;
    FDialogHandle: Integer;
    FTitle: string ;
    FOptions: TOptionsDir;
    FDisplayName: string;
    FFromDirectory: TFromDirectory;
    FDirectory: string;
    FFalse: string;
    FOnInit: TNotifyEvent;
    FPosition: TJvFolderPos;
    FOnChange: TJvDirChange;
    FPidl: TItemIDList;
  protected
  public
    constructor Create(AOwner: TComponent);override;
    procedure SetStatusText(Value: string);
    procedure SetOkEnabled(Value: boolean);
    property LastPidl: TItemIDList read FPidl write FPidl;

    function Execute: boolean;override;
  published
    property Directory: string read FDirectory write FFalse;
    property DisplayName: string read FDisplayName write FDisplayName;
    property Options: TOptionsDir read FOptions write FOptions default
      [odStatusAvailable, odNewDialogStyle];
    property Position: TJvFolderPos read FPosition write FPosition default fpDefault;
    property RootDirectory:  TFromDirectory read FFromDirectory write FFromDirectory;
    property Title: string read FTitle write FTitle;

    property OnInitialized: TNotifyEvent read FOnInit write FOnInit;
    property OnChange: TJvDirChange read FOnChange write FOnChange;
  end;

implementation

const
  BIF_BROWSEINCLUDEURLS  = $0080;
  BIF_BROWSEINCLUDEFILES = $4000;
  BIF_SHAREABLE          = $8000;
  BIF_RETURNFSANCESTORS  = $0008;
  BIF_EDITBOX            = $0010;
  BIF_VALIDATE           = $0020;
  BIF_NEWDIALOGSTYLE     = $0040;
  
{**************************************************}
constructor TJvBrowseFolder.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOptions := [odStatusAvailable, odNewDialogStyle];
  FHandle := GetParentForm(TControl(AOwner)).Handle;
end;
{**************************************************}
function lpfnBrowseProc(Wnd:HWND;uMsg:UINT;lParam,lpData:LPARAM):Integer stdcall;
var
 r,sr: TRect;
 st: string;
begin
  Result := 0;
  with TObject(lpData) as TJvBrowseFolder do
  begin
    case uMsg of
       BFFM_INITIALIZED :
         begin
           //Change the position of the dialog
           FDialogHandle := Wnd;
           if GetClientRect(Wnd,r) then
           begin
             SystemParametersInfo(SPI_GETWORKAREA,0,@sr,0);
             case FPosition of
                fpScreenCenter:
                  begin
                    r.Left := ((sr.Right-sr.Left-(r.Right-r.Left))div 2);
                    r.Top := (sr.Bottom-sr.top-(r.bottom-r.top))div 2;
                  end;
                fpFormCenter:
                  begin
                    GetWindowRect(FHandle,sr);
                    r.Left := sr.left+((sr.Right-sr.Left-(r.Right-r.Left))div 2);
                    r.Top := sr.top+(sr.Bottom-sr.top-(r.bottom-r.top))div 2;
                  end;
                fpTopLeft:
                  begin
                    r.Left := sr.left;
                    r.Top := sr.top;
                  end;
                fpTopRight:
                  begin
                    r.Top := sr.top;
                    r.Left := sr.Right-(r.right-r.left)-GetSystemMetrics(SM_CXFIXEDFRAME);
                  end;
                fpBottomLeft:
                  begin
                   r.Top := sr.Bottom-(r.bottom-r.top)-GetSystemMetrics(SM_CYCAPTION)--GetSystemMetrics(SM_CYFIXEDFRAME);
                   r.Left := sr.left;
                  end;
                fpBottomRight:
                  begin
                    r.Top := sr.Bottom-(r.bottom-r.top)-GetSystemMetrics(SM_CYCAPTION)-GetSystemMetrics(SM_CYFIXEDFRAME);
                    r.Left := sr.Right-(r.right-r.left)-GetSystemMetrics(SM_CXFIXEDFRAME);
                  end;
             end;
             SetWindowPos(Wnd,0,r.left,r.top,0,0,SWP_NOACTIVATE or SWP_NOSIZE or SWP_NOZORDER);
           end;

           //Change directory (if possible)
           if FDirectory<>'' then
           begin
             st := FDirectory;
             SendMessage(FDialogHandle,BFFM_SETSELECTION,Integer(true),Integer(st));
           end;

           //Call init event
           if Assigned(FOnInit) then
             FOnInit(TObject(lpData) as TJvBrowseFolder);
         end;
       BFFM_SELCHANGED :
         begin
           st:='';
           try
             if pointer(lparam)<>nil then
             begin
               SetLength(st, MAX_PATH);
               if SHGetPathFromIDList(PItemIDList(lparam),pchar(st)) then
                  SetLength(st,StrLen(PChar(st)));
             end;
           except
           end;
           if Assigned(FOnChange) then
             FOnChange(TObject(lpData) as TJvBrowseFolder, FDirectory);
         end;
    end;
  end;
end;
{**************************************************}
procedure TJvBrowseFolder.SetOkEnabled(Value: boolean);
begin
  if FDialogHandle<>0 then
    SendMessage(FDialogHandle,BFFM_ENABLEOK,0,integer(Value));
end;
{**************************************************}
procedure TJvBrowseFolder.SetStatusText(Value: string);
begin
  if FDialogHandle<>0 then
    SendMessage(FDialogHandle,BFFM_SETSTATUSTEXT,0,longint(pchar(Value)));
end;
{**************************************************}
function TJvBrowseFolder.Execute:boolean;
var
 path: array [0..MAX_PATH] of char;
 dspName: array [0..MAX_PATH] of char;
 BrowseInfo: TBrowseInfo;
 i: Integer;
 pidl: PItemIDList;
begin
  FDialogHandle:=0;
  result := false;

  if GetShellVersion<$00040000 then
  begin
    raise Exception.Create('Shell not compatible with BrowseForFolder');
    Exit;
  end;

  ZeroMemory(@BrowseInfo,SizeOf(BrowseInfo));

  if odBrowseForComputer in FOptions then
    BrowseInfo.ulFlags := BIF_BROWSEFORCOMPUTER;
  if odOnlyPrinters in FOptions then
    BrowseInfo.ulFlags := BIF_BROWSEFORPRINTER;
  if odNoBelowDomain in FOptions then
    BrowseInfo.ulFlags := BrowseInfo.ulFlags+BIF_DONTGOBELOWDOMAIN;
  if odSystemAncestorsOnly in FOptions then
    BrowseInfo.ulFlags := BrowseInfo.ulFlags+BIF_RETURNFSANCESTORS;
  if odFileSystemDirectoryOnly in FOptions then
    BrowseInfo.ulFlags := BrowseInfo.ulFlags+BIF_RETURNONLYFSDIRS;
  if odStatusAvailable in FOptions then
    BrowseInfo.ulFlags := BrowseInfo.ulFlags+BIF_STATUSTEXT;
  if (odIncludeFiles in FOptions) and (GetShellVersion>=$00040071) then
    BrowseInfo.ulFlags := BrowseInfo.ulFlags+BIF_BROWSEINCLUDEFILES;
  if (odIncludeUrls in FOptions) and (GetShellVersion>=$00050000) then
    BrowseInfo.ulFlags := BrowseInfo.ulFlags+BIF_BROWSEINCLUDEURLS;
  if (odEditBox in FOptions) and (GetShellVersion>=$00040071) then
    BrowseInfo.ulFlags := BrowseInfo.ulFlags+BIF_EDITBOX;
  if (odNewDialogStyle in FOptions) and (GetShellVersion>=$00050000) then
    BrowseInfo.ulFlags := BrowseInfo.ulFlags+BIF_NEWDIALOGSTYLE;
  if (odShareable in FOptions) and (GetShellVersion>=$00050000) then
    BrowseInfo.ulFlags := BrowseInfo.ulFlags+BIF_SHAREABLE;


  BrowseInfo.HwndOwner := Fhandle;
  i := 0;
  case FFromDirectory of
    fdRecycleBin   : i := CSIDL_BITBUCKET;
    fdControlPanel : i := CSIDL_CONTROLS;
    fdDesktop      : i := CSIDL_DESKTOP;
    fdDesktopDirectory : i := CSIDL_DESKTOPDIRECTORY;
    fdMyComputer   : i := CSIDL_DRIVES;
    fdFonts        : i := CSIDL_FONTS;
    fdNetHood      : i := CSIDL_NETHOOD;
    fdNetWork      : i := CSIDL_NETWORK;
    fdPersonal     : i := CSIDL_PERSONAL;
    fdPrinters     : i := CSIDL_PRINTERS;
    fdPrograms     : i := CSIDL_PROGRAMS;
    fdRecent       : i := CSIDL_RECENT;
    fdSendTo       : i := CSIDL_SENDTO;
    fdStartMenu    : i := CSIDL_STARTMENU;
    fdStartup      : i := CSIDL_STARTUP;
    fdTemplates    : i := CSIDL_TEMPLATES;
  end;
  if (i<>0) then
    SHGetSpecialFolderLocation(FHandle,i,BrowseInfo.pidlRoot);

  BrowseInfo.pszDisplayName := dspName;
  if FTitle='' then
    BrowseInfo.lpszTitle := nil
  else
    BrowseInfo.lpszTitle := pchar(FTitle);
  BrowseInfo.lpfn := @lpfnBrowseProc;
  BrowseInfo.lParam := LongInt(self);
  BrowseInfo.iImage := 0;

  try
    CoInitialize(nil);
    pidl := SHBrowseForFolder(BrowseInfo);
    if pidl<>nil then
    begin
      try
        FPidl := pidl^;
        if SHGetPathFromIDList(pidl,path) then
        begin
          if path<>nil then
            FDirectory := StrPas(path);
          result := true;
        end;
        FDisplayName := BrowseInfo.pszDisplayName;
        CoTaskMemFree(pidl);
        CoTaskMemFree(BrowseInfo.pidlRoot);
      except
      end;
    end;
    CoUninitialize();
  except
  end;
  FDialogHandle:=0;
end;
{**************************************************}
end.
