{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvCheckListBox.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvCheckListBox;

interface

{$ObjExportAll On}

uses
  Windows, Messages, SysUtils, Classes, Graphics, checklst, Controls, Forms,
  JvItemsSearchs;

type
  TJvCheckListBox = class(TCheckListbox)
  private
    FEffect: boolean;
    FOnMouseEnter: TNotifyEvent;
    FColor,FSaved: TColor;
    FOnMouseLeave: TNotifyEvent;
    FOnCtl3DChanged: TNotifyEvent;
    FOnParentColorChanged: TNotifyEvent;
    FOnSelectCancel: TNotifyEvent;
    Fover: boolean;
    FMaxWidth: Integer;
    FScroll: boolean;
    FOnHScroll: TNotifyEvent;
    FOnVScroll: TNotifyEvent;
    FItemSearchs: TJvItemsSearchs;
    procedure SetHScroll(const Value: boolean);
    procedure RefreshH;
    procedure SetEffect(const Value: boolean);
  protected
    procedure WMHScroll(var Message: TWMHScroll); message WM_HSCROLL;
    procedure WMVScroll(var Message: TWMVScroll); message WM_VSCROLL;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorCHANGED;
    procedure SelectCancel(var Message: TMessage);message LBN_SELCANCEL;
    procedure CreateParams(var Params: TCreateParams);override;
    procedure WndProc(var Message : TMessage); override;
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;

    function SearchExactString(Value: string; CaseSensitive: Boolean = true):Integer;
    function SearchPrefix(Value: string; CaseSensitive: Boolean = true):Integer;
    function SearchSubString(Value: string; CaseSensitive: Boolean = true):Integer;
    function DeleteExactString(Value: string; All: Boolean;
      CaseSensitive: Boolean = true):Integer;

    procedure SelectAll;
    procedure UnselectAll;
    procedure InvertSelection;
    procedure CheckAll;
    procedure UnCheckall;
    procedure InvertCheck;
    function GetChecked:TStringList;
    function GetUnChecked:TStringList;
    procedure DeleteSelected;
    procedure SaveToFile(FileName: TFileName);
    procedure LoadFromFile(FileName: TFileName);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToStream(Stream: TStream);
  published
    property HotTrack: boolean read FEffect write SetEffect default false;
    property HorScrollbar:boolean read FScroll write SetHScroll default true;
    property HintColor: TColor read FColor write FColor default clInfoBk;

    property OnMouseEnter: TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave: TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property OnCtl3DChanged: TNotifyEvent read FonCtl3DChanged write FonCtl3DChanged;
    property OnParentColorChange: TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
    property OnSelectCancel: TNotifyEvent read FOnSelectCancel write FOnSelectCancel;
    property OnVerticalScroll: TNotifyEvent read FOnVScroll write FOnVScroll;
    property OnHorizontalScroll: TNotifyEvent read FOnHScroll write FOnHScroll;
  end;

implementation

type
  //Used for the load/save methods
  TCheckListRecord = record
    Checked: boolean;
    StringSize: Integer;
  end;

{**************************************************}
constructor TJvCheckListBox.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FMaxWidth := 0;
  FEffect:=false;
  FColor:=clInfoBk;
  FOver:=false;
  FScroll:=true;
  Controlstyle := Controlstyle + [csAcceptsControls];
  FItemSearchs := TJvItemsSearchs.Create;
end;
{**************************************************}
destructor TJvCheckListBox.Destroy;
begin
  FItemSearchs.Free;
  inherited;
end;
{**************************************************}
procedure TJvCheckListBox.CreateParams(var Params: TCreateParams);
begin
  inherited;
  with Params do
    if FScroll then
      Style := Style or WS_HSCROLL
    else
      Style := Style xor WS_HSCROLL;
end;
{***********************************************}
procedure TJvCheckListBox.RefreshH;
var
 i: Integer;
 ItemWidth : Word;
begin
  FMaxWidth := 0;
  for i:=0 to self.Items.Count-1 do
  begin
    ItemWidth := self.Canvas.TextWidth(Items[i]+' ');
    if FMaxWidth<ItemWidth then
      FMaxWidth:=ItemWidth;
  end;
  SetHScroll(FScroll);
end;
{***********************************************}
procedure TJvCheckListBox.WndProc(var Message : TMessage);
var
 ItemWidth:Word;
begin
  case Message.Msg of
    LB_ADDstring, LB_INSERTSTRING :
       begin
         ItemWidth := Canvas.TextWidth(StrPas(PChar(Message.lParam))+' ');
         if FMaxWidth < ItemWidth then
           FMaxWidth := ItemWidth;
         SetHScroll(FScroll);
       end;
    LB_DELETESTRING :
       begin
         ItemWidth := Canvas.TextWidth(Items[Message.wParam]+' ');
         if ItemWidth = FMaxWidth then
         begin
           inherited;
           RefreshH;
           Exit;
         end;
       end;
    LB_RESETCONTENT :
       begin
         FMaxWidth := 0;
         SetHScroll(FScroll);
       end;
    WM_SETFONT :
       begin
         inherited;
         Canvas.Font.Assign(Self.Font);
         RefreshH;
         Exit;
       end;
  end;
  inherited;
end;
{***********************************************}
procedure TJvCheckListBox.SetHScroll(const Value: boolean);
begin
  FScroll := Value;
  if FScroll then
    SendMessage(Handle, LB_SETHORIZONTALEXTENT, FMaxWidth, 0);
end;
{***********************************************}
procedure TJvCheckListBox.CMCtl3DChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonCtl3DChanged) then
    FonCtl3DChanged(self);
end;
{**************************************************}
procedure TJvCheckListBox.CMParentColorChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(FonParentColorChanged) then
    FonParentColorChanged(Self);
end;
{**************************************************}
procedure TJvCheckListBox.CMMouseEnter(var Message: TMessage);
begin
  if not FOver then
  begin
    FSaved := Application.HintColor;
    Application.HintColor := FColor;
    if FEffect then
      ctl3d := true;
    FOver := true;
  end;
  if Assigned(FonMouseEnter) then
    FonMouseEnter(self);
end;
{**************************************************}
procedure TJvCheckListBox.WMHScroll(var Message: TWMHScroll);
begin
  inherited;
  if Assigned(FOnHScroll) then
    FOnHScroll(self);
end;
{**************************************************}
procedure TJvCheckListBox.WMVScroll(var Message: TWMVScroll);
begin
  inherited;
  if Assigned(FOnVScroll) then
    FOnVScroll(self);
end;
{**************************************************}
procedure TJvCheckListBox.CMMouseLeave(var Message: TMessage);
begin
  if FOver then
  begin
    Application.HintColor := FSaved;
    if FEffect then
      ctl3d := false;
    FOver := false;
  end;
  if Assigned(FonMouseLeave) then
    FonMouseLeave(self);
end;
{**************************************************}
function TJvCheckListBox.SearchExactString(Value: string;
  CaseSensitive: Boolean): Integer;
begin
  result := FItemSearchs.SearchExactString(Items,Value,CaseSensitive);
end;
{**************************************************}
procedure TJvCheckListBox.SetEffect(const Value: boolean);
begin
  FEffect := Value;
  if Feffect then
    ctl3d := false;
end;
{**************************************************}
function TJvCheckListBox.SearchPrefix(Value: string; CaseSensitive: Boolean):Integer;
begin
  result := FItemSearchs.SearchPrefix(Items,Value,CaseSensitive);
end;
{**************************************************}
procedure TJvCheckListBox.SelectCancel(var Message: TMessage);
begin
  if Assigned(FOnSelectCancel) then
    FOnSelectCancel(self);
end;
{**************************************************}
function TJvCheckListBox.SearchSubString(Value: string;
  CaseSensitive: Boolean): Integer;
begin
  result := FItemSearchs.SearchSubString(Items,Value,CaseSensitive);
end;
{**************************************************}
function TJvCheckListBox.DeleteExactString(Value: string; All: Boolean;
  CaseSensitive: Boolean): Integer;
begin
  result := FItemSearchs.DeleteExactString(Items,Value,CaseSensitive);
end;
{**************************************************}
procedure TJvCheckListBox.SelectAll;
var
 i:Integer;
begin
  if MultiSelect = false then
    exit;
  for i:=0 to Items.Count-1 do
     Selected[i] := true;
end;
{**************************************************}
procedure TJvCheckListBox.UnselectAll;
var
 i:Integer;
begin
  if MultiSelect = false then
    exit;
  for i:=0 to Items.Count-1 do
    Selected[i] := false;
end;
{**************************************************}
procedure TJvCheckListBox.InvertSelection;
var
 i:Integer;
begin
  if MultiSelect = false then
    exit;
  for i:=0 to Items.Count-1 do
    Selected[i] := not(Selected[i]);
end;
{**************************************************}
procedure TJvCheckListBox.CheckAll;
var
 i: Integer;
begin
  for i:=0 to Items.Count-1 do
    Checked[i] := true;
end;
{**************************************************}
procedure TJvCheckListBox.UnCheckall;
var
 i: Integer;
begin
  for i:=0 to Items.Count-1 do
    Checked[i] := false;
end;
{**************************************************}
procedure TJvCheckListBox.InvertCheck;
var
 i: Integer;
begin
  for i:=0 to Items.Count-1 do
    Checked[i] := not(Checked[i]);
end;
{**************************************************}
function TJvCheckListBox.GetChecked:TStringList;
var
 i: Integer;
begin
  result := TStringList.Create;
  for i:=0 to Items.Count-1 do
   if (Checked[i]) then
      result.AddObject(Items[i],Items.Objects[i]);
end;
{**************************************************}
function TJvCheckListBox.GetUnChecked:TstringList;
var
 i:Integer;
begin
  result := TStringList.Create;
  for i:=0 to Items.Count-1 do
    if (not(Checked[i])) then
      result.AddObject(Items[i],Items.Objects[i]);
end;
{**************************************************}
procedure TJvCheckListBox.LoadFromFile(FileName: TFileName);
var
 Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName,fmOpenRead or fmShareDenyWrite);
  LoadFromStream(Stream);
  Stream.Free;
end;
{**************************************************}
procedure TJvCheckListBox.LoadFromStream(Stream: TStream);
var
 CheckLst: TCheckListRecord;
 st: string;
 buf: array[0..1024] of char;
begin
  Items.Clear;
  while (Stream.Position+SizeOf(TCheckListRecord)<=Stream.Size) do
  begin
    Stream.Read(CheckLst,SizeOf(TCheckListRecord));
    if ((Stream.Position+CheckLst.StringSize)<=Stream.Size) then
    begin
      Stream.Read(buf,CheckLst.StringSize);
      buf[CheckLst.StringSize] := #0;
      st := buf;
      Checked[Items.Add(st)] := CheckLst.Checked;
    end;
  end;   
end;
{**************************************************}
procedure TJvCheckListBox.SaveToFile(FileName: TFileName);
var
 Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName,fmCreate or fmShareExclusive);
  SaveToStream(Stream);
  Stream.Free;
end;
{**************************************************}
procedure TJvCheckListBox.SaveToStream(Stream: TStream);
var
 CheckLst: TCheckListRecord;
 buf: array[1..1024] of char;
 i,j: Integer;
begin
  for i:=0 to Items.Count-1 do
  begin
    CheckLst.Checked := Checked[i];
    CheckLst.StringSize := Length(Items[i]);
    Stream.Write(CheckLst,SizeOf(TCheckListRecord));
    for j:=1 to Length(Items[i]) do
      buf[j] := Items[i][j];
    Stream.Write(buf,CheckLst.StringSize);
  end;
end;
{**************************************************}
procedure TJvCheckListBox.DeleteSelected;
var
 i:Integer;
begin
  if MultiSelect then
  begin
   for i:=Items.Count-1 downto 0 do
    if Selected[i] then
      Items.Delete(i);
  end
  else
    if ItemIndex<>-1 then
    begin
      i := ItemIndex;
      Items.Delete(i);
      if i>0 then dec(i);
      if Items.Count>0 then
        ItemIndex := i;
    end;
end;
{**************************************************}
end.
