{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvClipboardViewer.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvClipboardViewer;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  JvTypes, ClipBrd;

type
  TJvClipboardViewer = class(TComponent)
  private
    FHandle  : THandle;
    FNextCB  : HWND;
    FOnText  : TOnText;
    FOnImage : TOnImage;
    procedure UpdateClip(var msg: TWMDrawClipBoard); message WM_DRAWCLIPBOARD;
  protected
  public
    procedure WndProc(var Msg: TMessage);
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    procedure EmptyClipboard;
    property OnImage:TOnImage read FOnImage write FOnImage;
    property OnText:TOnText read FOnText write FOnText;
  end;

implementation

{**************************************************}
constructor TJvClipboardViewer.Create(AOwner: TComponent);
begin
   inherited;
   FHandle:=AllocateHWND(WndProc);
   FNextCB := SetClipboardViewer(FHandle);
   setclipboardviewer(FHandle);
end;
{**************************************************}
destructor TJvClipboardViewer.Destroy;
begin
   DeallocateHWnd(FHandle);
   ChangeClipboardChain(FHandle, FNextCB);
   inherited;
end;
{**************************************************}
procedure TJvClipboardViewer.EmptyClipboard;
begin
   openclipboard(Application.handle);
   emptyclipboard;
   closeclipboard;
end;
{**************************************************}
procedure TJvClipboardViewer.UpdateClip(var msg: TWMDrawClipBoard);
var
   bitmap:Tbitmap;
begin
   inherited;
   if Clipboard.HasFormat(CF_BITMAP) then
   begin
      bitmap:=nil;
      try
        bitmap:=Tbitmap.Create;
        bitmap.assign(clipboard);
        if Assigned(FOnImage) then
           FOnImage(self,bitmap);
      finally
        bitmap.free;
      end;
   end
   else
      if Assigned(FOnText) then
         FOnText(self,ClipBoard.asText);
   msg.Result := 0;
end;
{**************************************************}
procedure TJvClipboardViewer.WndProc(var Msg: TMessage);
begin
   if (Msg.Msg = WM_DRAWCLIPBOARD) then
     UpdateClip(TWMDrawClipBoard(Msg))
   else
     Msg.Result:=DefWindowProc(FHandle, Msg.Msg, Msg.wParam, Msg.lParam);
end;
{**************************************************}
end.

