{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvControlPanel.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvControlPanel;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, JvTypes, JvButton, Menus, JvDirectories, Cpl, JvFunctions,
  ShellApi;

type
  TJvControlPanel = class(TJvButton)
  private
    FPopup:TPopupMenu;
    FDirs:TJvDirectories;
    FOnUrl:TOnLinkClick;
    FLeft,FTop:Integer;
    procedure AddToPopup(Item:TMenuItem;path:string);
    procedure UrlClick(Sender: TObject);
  public
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Click; override;
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property OnLinkClick:TOnLinkClick read FOnUrl write FOnUrl;
  end;

implementation

resourcestring
   RC_CplAddress       =     'CPlApplet';

{*******************************************************}
function GetNameCpl(path:string):string;
var
  h:thandle;
  CplApplet:TCplApplet;
  NewCplInfo:PNewCplInfo;
begin
   result:='';
   h:=loadlibrary(pchar(path));
   if h<>0 then
   begin
      @CplApplet:=GetProcAddress(h,PChar(RC_CplAddress));
      if @CplApplet<>nil then
      begin
         NewCplInfo:=new(PNewCplInfo);
         NewCplInfo.szName:='';
         CplApplet(0,CPL_NEWINQUIRE,0,longint(NewCplInfo));
         if NewCplInfo<>nil then
            result:=NewcplInfo^.szName;
      end;
      freelibrary(h);
   end;
end;
{*******************************************************}
procedure TJvControlPanel.AddToPopup(Item: TMenuItem; path: string);
var
   t:TSearchRec;
   res,i:Integer;
   it:TMenuItem;
   ts:TstringList;
   st:string;
   w:word;
begin
   ts:=TstringList.Create;
   res:=FindFirst(path+'*.cpl',faAnyFile,t);
   while res=0 do
   begin
      if (t.name<>'.')and(t.name<>'..') then
      begin
         st:=GetNameCpl(path+t.name);
         if st='' then
         begin
            st:=t.Name;
            i:=length(st);
            while (i>0) and (st[i]<>'.') do dec(i);
              st:=Copy(st,1,i-1);
         end;
         ts.Add(st+'%'+t.name);
       end;
       res:=FindNext(t);
   end;
   FindClose(t);
   ts.sort;

   for res:=0 to ts.Count-1 do
   begin
      it:=TMenuItem.Create(self);
      it.caption:=Copy(ts[res],1,pos('%',ts[res])-1);
      it.OnClick:=UrlClick;
      it.Hint:=path+Copy(ts[res],pos('%',ts[res])+1,length(ts[res]));;
      w:=0;
      it.Bitmap.assign(IconToBitmap(ExtractAssociatedIcon(Application.handle,pchar(it.hint),w)));
      it.Bitmap.TransparentMode:=tmAuto;
      item.add(it);
      Application.ProcessMessages;
   end;
   ts.free;
end;
{*******************************************************}
procedure TJvControlPanel.Click;
begin
   inherited;
   Fpopup.Popup(fleft+self.left,ftop+self.top+self.height+20);
end;
{*******************************************************}
constructor TJvControlPanel.Create(AOwner: TComponent);
begin
   inherited;
   FLeft := GetParentForm(TControl(AOwner)).Left;
   FTop := GetParentForm(TControl(AOwner)).Top;
   FDirs := TJvDirectories.Create(self);
   FPopup := TPopupMenu.Create(self);
end;
{*******************************************************}
procedure TJvControlPanel.CreateParams(var Params: TCreateParams);
var
  st:string;
begin
   inherited;
   if not(csDesigning in componentstate) then
   begin
      while FPopup.items.count>0 do
            Fpopup.items.Delete(0);
      st:=FDirs.SystemDirectory;
      if st[length(st)]<>'\' then st:=st+'\';
      AddToPopup(TmenuItem(FPopup.items),st);
      FTop:=(self.Owner as TForm).top;
      FLeft:=(Self.Owner as TForm).Left;
      self.PopupMenu:=FPopup;
   end;
end;
{*******************************************************}
destructor TJvControlPanel.Destroy;
var
   i:Integer;
begin
   FDirs.free;
   for i:=0 to FPopup.Items.Count-1 do
        Fpopup.items[i].Bitmap.FreeImage;
   FPopup.free;
   inherited;
end;
{*******************************************************}
procedure TJvControlPanel.UrlClick(Sender: TObject);
begin
   if Assigned(FOnUrl) then FOnUrl(self,(Sender as TMenuItem).hint);
end;
{*******************************************************}
end.
