{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvHSetup.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvHSetup;

{*******************************************************}
{   This unit is an interface to the Setup (setupapi)   }
{   Imported disk prompting and eror handling functions }
{*******************************************************}

interface

uses
    WinTypes, WinProcs, Dialogs;


const
    IDF_NOBROWSE        = $00000001;
    IDF_NOSKIP          = $00000002;
    IDF_NODETAILS       = $00000004;
    IDF_NOCOMPRESSED    = $00000008;
    IDF_CHECKFIRST      = $00000100;
    IDF_NOBEEP          = $00000200;
    IDF_NOFOREGROUND    = $00000400;
    IDF_WARNIFSKIP      = $00000800;
    IDF_OEMDISK         = $80000000;

    DPROMPT_SUCCESS         = 0;
    DPROMPT_CANCEL          = 1;
    DPROMPT_SKIPFILE        = 2;
    DPROMPT_BUFFERTOOSMALL  = 3;
    DPROMPT_OUTOFMEMORY     = 4;

    SPFILENOTIFY_STARTQUEUE    = $00000001;
    SPFILENOTIFY_ENDQUEUE      = $00000002;
    SPFILENOTIFY_STARTSUBQUEUE = $00000003;
    SPFILENOTIFY_ENDSUBQUEUE   = $00000004;
    SPFILENOTIFY_STARTDELETE   = $00000005;
    SPFILENOTIFY_ENDDELETE     = $00000006;
    SPFILENOTIFY_DELETEERROR   = $00000007;
    SPFILENOTIFY_STARTRENAME   = $00000008;
    SPFILENOTIFY_ENDRENAME     = $00000009;
    SPFILENOTIFY_RENAMEERROR   = $0000000a;
    SPFILENOTIFY_STARTCopy     = $0000000b;
    SPFILENOTIFY_ENDCopy       = $0000000c;
    SPFILENOTIFY_CopyERROR     = $0000000d;
    SPFILENOTIFY_NEEDMEDIA     = $0000000e;
    SPFILENOTIFY_QUEUESCAN     = $0000000f;

    SPFILENOTIFY_CABINETINFO     = $00000010;
    SPFILENOTIFY_FILEINCABINET   = $00000011;
    SPFILENOTIFY_NEEDNEWCABINET  = $00000012;
    SPFILENOTIFY_FILEEXTRACTED   = $00000013;
    SPFILENOTIFY_FILEOPDELAYED   = $00000014;

    SPFILENOTIFY_LANGMISMATCH    = $00010000;
    SPFILENOTIFY_TARGETEXISTS    = $00020000;
    SPFILENOTIFY_TARGETNEWER     = $00040000;

    FILEOP_Copy     = 0;
    FILEOP_RENAME   = 1;
    FILEOP_DELETE   = 2;
    FILEOP_ABORT    = 0;
    FILEOP_DOIT     = 1;
    FILEOP_SKIP     = 2;
    FILEOP_RETRY    = FILEOP_DOIT;
    FILEOP_NEWPATH  = 4;


type
    PSP_FILE_CALLBACK = function (Context:Pointer;Notification,Param1:Integer;var Param2:integer):Integer;stdcall;
    PPSP_FILE_CALLBACK = PSP_FILE_CALLBACK;

    PFILE_IN_CABINET_INFO = ^FILE_IN_CABINET_INFO;
    FILE_IN_CABINET_INFO = record
      NameInCabinet : PChar;
      FileSize : DWORD;
      Win32Error : DWORD;
      DosDate : WORD;
      DosTime : WORD;
      DosAttribs : WORD;
      FullTargetName : array[0..255] of Char;
    end;

    PCabinet_Info = ^Cabinet_Info;
    Cabinet_Info = record
        CabinetPath : pchar;
        CabinetFile : pchar;
        DiskName : pchar;
        Id : Shortint;
        CabinetNumber : Shortint;
    end;

    PFilePath = ^TFilePath;
    TFilePath = record
        Target:PChar;
        Source:PChar;
        ErrorCode:Integer;
        Flags:DWord;
    end;

TSetupPromptForDisk = function (
    Handle:HWND;             // parent window of the dialog box
    DialogTitle:pchar;       // optional, title of the dialog box
    DiskName:pchar;          // optional, name of disk to insert
    PathToSource:pchar;      // optional, expected source path
    FileSought:pchar;        // name of file needed
    TagFile:pchar;           // optional, source media tag file
    DiskPromptstyle:DWord;   // specifies dialog box behavior
    PathBuffer:pchar;        // receives the source location
    PathBufferSize:DWord;    // size of the supplied buffer
    PathRequiredSize:PDWord  // optional, buffer size needed
   ):Integer;stdcall;

TSetupCopyError = function (
    Handle:HWND;             // parent window for this dialog box
    DialogTitle:Pchar;       // optional, title for this dialog box
    DiskName:Pchar;          // optional, name of disk to insert
    PathToSource:Pchar;      // failed source path
    Sourcefile:PChar;        // source file of Copy error
    TargetPathFile:Pchar;    // optional, target file of Copy error
    Win32ErrorCode:Integer;  // error information
    style:DWord;             // dialog box formatting and display
    PathBuffer:PChar;        // optional, receives new path info
    PathBufferSize:DWord;    // size of supplied buffer
    PathRequiredSize:PDWord  // optional, buffer size needed
   ):Integer;stdcall;

TSetupDeleteError = function (
    Handle:HWND;             // parent window for this dialog box
    DialogTitle:Pchar;       // optional, title for this dialog box
    FileName:PChar;          // file that caused the delete error
    Win32ErrorCode:Integer;  // specifies the error that occurred
    style:DWord              // specifies formatting for the dialog box
   ):Integer;stdcall;

TSetupRenameError = function (
    Handle:HWND;             // parent window for this dialog box
    DialogTitle:Pchar;	     // optional, title for this dialog box
    SourceFile:PChar;	     // source file of the rename error
    TargetFile:Pchar;	     // target file of the rename error
    Win32ErrorCode:Integer;  // the error encountered
    style:DWord	             // specifies formatting for this dialog box
   ):Integer;stdcall;

TSetupIterateCabinet = function (
    CabinetFile:pchar;          // name of the cabinet file
    Reserved:DWord;             // this parameter is not used
    MsgHandle:PPSP_FILE_CALLBACK;// callback routine to use
    Context:pointer             // callback routine context
   ):boolean;stdcall;

implementation

end.
