{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvMessageBox.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvMessageBox;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  JvBaseDlg, JvTypes;

type
  TJvMessageBox = class(TComponent)
  private
    FText: string;
    FTitle: string;
    FDisplay: TButtonDisplay;
    FOptions: TButtonOptions;
    Fstyle: TButtonstyle;
    FDefault: TDefault;
    FModality: TModality;
  protected
  public
    constructor Create(AOwner: TComponent);override;
  published
    property Text:string read FText write FText;
    property Title:string read FTitle write FTitle;
    property style:TButtonstyle read Fstyle write Fstyle default bsOkCancel;
    property Display:TButtonDisplay read FDisplay write FDisplay default bdIconQuestion;
    property DefaultButton:TDefault read FDefault write FDefault default dbButton1;
    property Modality:TModality read FModality write FModality default bmApplModal;
    property Options:TButtonOptions read FOptions write FOptions;
    function Execute:TButtonResult;
  end;

implementation

{**************************************************}
constructor TJvMessageBox.Create(AOwner: TComponent);
begin
   inherited;
   Fstyle:=bsOkCancel;
   FDisplay:=bdIconQuestion;
   FDefault:=dbButton1;
   Fmodality:=bmApplModal
end;
{**************************************************}
function TJvMessageBox.Execute: TButtonResult;
var
   Flags:Integer;

begin
   Flags:=0;
   case Fstyle of
        bsAbortRetryIgnore : Flags:=MB_AbortRetryIgnore;
        bsOk : Flags:=MB_OK;
        bsOkCancel : Flags:=MB_OkCancel;
        bsRetryCancel : Flags:=MB_RetryCancel;
        bsYesNo : Flags:=MB_YesNo;
        bsYesNoCancel : Flags:=MB_YesNoCancel;
   end;
   case FDisplay of
        bdIconExclamation : Flags:=Flags+MB_IconExclamation;
        bdIconWarning : Flags:=Flags+MB_IconWarning;
        bdIconInformation : Flags:=Flags+MB_IconInformation;
        bdIconAsterisk : Flags:=Flags+MB_IconAsterisk;
        bdIconQuestion : Flags:=Flags+MB_IconQuestion;
        bdIconStop : Flags:=Flags+MB_IconStop;
        bdIconError : Flags:=Flags+MB_IconError;
        bdIconHand : Flags:=Flags+MB_IconHand;
   end;
   case FDefault of
        dbButton1 : Flags:=Flags+MB_DEFBUTTON1;
        dbButton2 : Flags:=Flags+MB_DEFBUTTON2;
        dbButton3 : Flags:=Flags+MB_DEFBUTTON3;
        dbButton4 : Flags:=Flags+MB_DEFBUTTON4;
   end;
   case FModality of
        bmApplModal : Flags:=Flags+MB_ApplModal;
        bmSystemModal : Flags:=Flags+MB_SystemModal;
        bmTaskModal : Flags:=Flags+MB_TaskModal;
   end;
   if boDefaultDesktopOnly in FOptions then Flags:=Flags+MB_Default_Desktop_Only;
   if boHelp in FOptions then Flags:=Flags+MB_Help;
   if boRight in FOptions then Flags:=Flags+MB_Right;
   if boRtlReading in FOptions then Flags:=Flags+MB_RtlReading;
   if boSetForeground in FOptions then Flags:=Flags+MB_SetForeground;
   if boTopMost in FOptions then Flags:=Flags+MB_TopMost;

   case MessageBox(TForm(self.Owner).Handle,Pchar(FText),PChar(FTitle),Flags) of
        IDABORT : result:=brAbort;
        IDCancel : result:=brCancel;
        IDIgnore : result:=brIgnore;
        IDNo : result:=brNo;
        IDOk : result:=brOk;
        iDRetry : result:=brRetry;
        IDYes : result:=brYes;
        else result:=brAbort;
   end;
end;
{**************************************************}
end.
