{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvMru.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvMru;

{$ObjExportAll On} 

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  JvHMru;

Type
  TJvHKey = (hkClassesRoot, hkCurrentUser, hkLocalMachine, hkUsers, hkCurrentConfig);
  {$EXTERNALSYM TJvHKey}
  TJvDataType = (dtstring, dtBinary);
  {$EXTERNALSYM TJvDataType}
  TOnEnumData = procedure (Sender: TObject;Data:pointer;Size:Integer;Index:integer) of object;
  TOnEnumText = procedure (Sender: TObject;Value:string;Index:integer) of object;

  TJvMruList = class(TComponent)
  private
    FMax: Integer;
    FSubKey: string;
    FKey: TJvHKey;
    FList:THandle;
    FType: TJvDataType;
    FOnEnumData: TOnEnumData;
    FOnEnumText: TOnEnumText;
    procedure SetKey(const Value: TJvHKey);
    procedure SetMax(const Value: integer);
    procedure SetSubKey(const Value: string);
    procedure SetType(const Value: TJvDataType);
  protected
    procedure ReCreateList;
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property RootKey:TJvHKey read FKey write SetKey default hkCurrentUser;
    property SubKey:string read FSubKey write SetSubKey;
    property MaxItems:Integer read FMax write SetMax default 10;
    property DataType:TJvDataType read FType write SetType default dtstring;
    property OnEnumText:TOnEnumText read FOnEnumText write FOnEnumText;
    property OnEnumData:TOnEnumData read FOnEnumData write FOnEnumData;
    function Addstring(Value:string):boolean;
    function AddData(Value:Pointer;Size:integer):boolean;
    function GetItemsCount:Integer;
    function EnumItems:boolean;
    function GetMostRecentItem:boolean;
    function GetItem(Index:integer=0):boolean;
    function Findstring(Value:string):Integer;
    function FindData(Value:Pointer;Size:integer):Integer;
    function Deletestring(Index:integer=0):boolean;
  end;

  EMruException = class(Exception);

implementation

resourcestring
   RC_ErrorCreatingMRU     =      'Unable to create the MRU';

{**************************************************}
function TJvMruList.AddData(Value: Pointer; Size: integer): boolean;
begin
   if FList<>0 then
     result:=AddMruData(FList,Value,Size)<>-1
   else
     result:=false;
end;
{**************************************************}
function TJvMruList.Addstring(Value: string): boolean;
var
  p:PChar;
begin
   if FList<>0 then
   begin
      p:=PChar(Value);
      result:=AddMrustring(FList,p)<>-1
   end
   else
     result:=false;
end;
{**************************************************}
constructor TJvMruList.Create(AOwner: TComponent);
begin
   inherited;
   FList:=0;
   FMax:=10;
   FType:=dtstring;
   FKey:=hkCurrentUser;

   ReCreateList;
end;
{**************************************************}
function TJvMruList.Deletestring(Index: integer): boolean;
begin
   if FList<>0 then
      result:=DelMRUstring(FList,Index)
   else
      result:=false;
end;
{**************************************************}
destructor TJvMruList.Destroy;
begin
   if FList<>0 then
      FreeMruList(FList);
   inherited;
end;
{**************************************************}
function TJvMruList.EnumItems:boolean;
var
  st,st2:string;
  i:Integer;
  p:pointer;
  index:Integer;
begin
   result:=false;
   if FList=0 then
      exit;

   if FType=dtstring then
   begin
      SetLength(st,256);  
      i:=0;Index:=0;
      while (i<>-1) do
      begin
         i:=EnumMRUList(FList,Index,PChar(st),256);
         if i<>-1 then
         begin
            result:=true;
            if (Assigned(FOnEnumText)) then
            begin
               st2:=st;
               SetLength(st2,i);
               FOnEnumText(self,st2,Index);
            end;
         end;
         inc(Index);
      end;
   end
   else
   begin
      Index:=0;i:=0;
      while (i<>-1) do
      begin
         p:=AllocMem(1024);
         i:=EnumMRUList(FList,Index,p,1024);

         if i=1024 then
         begin
            FreeMem(p);
            p:=AllocMem(64000);
            i:=EnumMRUList(FList,0,p,64000);
         end;

         if i<>-1 then
         begin
            result:=true;
            if (Assigned(FonEnumData)) then
               FOnEnumData(self,p,i,Index);
         end;
         inc(Index);
      end;
   end;
end;
{**************************************************}
function TJvMruList.FindData(Value: Pointer; Size: integer): Integer;
begin
   if FList<>0 then
      result:=FindMRUData(FList,Value,Size,nil)
   else
      result:=-1;
end;
{**************************************************}
function TJvMruList.Findstring(Value: string): Integer;
begin
   if FList<>0 then
      result:=FindMRUstring(FList,PChar(Value),nil)
   else
      result:=-1;
end;
{**************************************************}
function TJvMruList.GetItem(Index: integer):boolean;
var
  st,st2:string;
  i:Integer;
  p:pointer;
begin
   result:=false;
   if FList=0 then
      exit;

   if FType=dtstring then
   begin
      SetLength(st,256);
      i:=EnumMRUList(FList,Index,PChar(st),256);
      if i<>-1 then
      begin
         result:=true;
         if (Assigned(FOnEnumText)) then
         begin
            st2:=st;
            SetLength(st2,i);
            FOnEnumText(self,st2,Index);
         end;
      end;
   end
   else
   begin
      p:=AllocMem(1024);
      i:=EnumMRUList(FList,Index,p,1024);

      if i=1024 then
      begin
         FreeMem(p);
         p:=AllocMem(64000);
         i:=EnumMRUList(FList,0,p,64000);
      end;

      if i<>-1 then
      begin
         result:=true;
         if (Assigned(FonEnumData)) then
         begin
            FOnEnumData(self,p,i,Index);
         end;
      end;
   end;
end;
{**************************************************}
function TJvMruList.GetItemsCount: Integer;
begin
   if FList<>0 then
      result:=EnumMRUList(FList,-1,nil,0)
   else
      result:=0;
end;
{**************************************************}
function TJvMruList.GetMostRecentItem:boolean;
begin
   result:=GetItem(0);
end;
{**************************************************}
procedure TJvMruList.ReCreateList;
var
  FLst:TMruList;
begin
   if FList<>0 then
      FreeMruList(FList);

   if FSubKey<>'' then
   begin
      FLst.cbSize:=SizeOf(FList);
      FLst.nMaxItems:=FMax;
      case FType of
          dtstring : begin
                        FLst.dwFlags:=MRUF_string_LIST;
                        FLst.lpfnComparestring:=nil;
                     end;
          dtBinary : begin
                        FLst.dwFlags:=MRUF_BINARY_LIST;
                        FLst.lpfnCompareData:=nil;
                     end;
      end;
      case FKey of
          hkClassesRoot   : FLst.hKey:=HKEY_CLASSES_ROOT;
          hkCurrentUser   : FLst.hKey:=HKEY_CURRENT_USER;
          hkLocalMachine  : FLst.hKey:=HKEY_LOCAL_MACHINE;
          hkUsers         : FLst.hKey:=HKEY_USERS;
          hkCurrentConfig : FLst.hKey:=HKEY_CURRENT_CONFIG;
      end;
      FLst.lpszSubKey:=PChar(FSubKey);

      FList := CreateMRUList(@FLst);
      if FList=0 then
         Raise EMruException.Create(RC_ErrorCreatingMRU);
   end;
end;
{**************************************************}
procedure TJvMruList.SetKey(const Value: TJvHKey);
begin
   if Value<>FKey then
   begin
      FKey := Value;
      ReCreateList;
   end;
end;
{**************************************************}
procedure TJvMruList.SetMax(const Value: integer);
begin
   if Value<>FMax then  
   begin
      FMax := Value;
      ReCreateList;
   end;
end;
{**************************************************}
procedure TJvMruList.SetSubKey(const Value: string);
begin
   if Value<>FSubKey then
   begin
      FSubKey := Value;
      ReCreateList;
   end;
end;
{**************************************************}
procedure TJvMruList.SetType(const Value: TJvDataType);
begin
   if Value<>FType then
   begin
      FType := Value;
      ReCreateList;
   end;
end;
{**************************************************}

end.

