{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvNagScreen.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvNagScreen;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, 
  JvBaseDlg, JvTypes, ExtCtrls;

type
  TJvNagScreen = class(TJvCommonDialogP)
  private
    Fpict:TImage;
    FTImer:TTimer;
    FTime: cardinal;
    FPicture: TBitmap;
    FForm:TForm;
    FDisappear: boolean;
    FOnAfter: TNotifyEvent;
    FOnBefore: TNotifyEvent;
    procedure Creer(Sender: TObject);
    procedure Cacher(Sender: TObject);
    procedure SetPicture(const Value: TBitmap);
  protected
    procedure Loaded;override;
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property Picture:TBitmap read FPicture write SetPicture;
    property TimeToStay:cardinal read FTime write FTime default 5000;
    property DisappearOnClick:boolean read FDisappear write FDisappear default true;
    procedure Execute;override;
    property OnBeforeNag:TNotifyEvent read FOnBefore write FOnBefore;
    property OnAfterNag:TNotifyEvent read FOnAfter write FOnAfter;
  end;

implementation

{**************************************************}
procedure TJvNagScreen.Cacher(Sender: TObject);
begin
  try
    FTimer.Enabled:=false;
    FForm.Close;
    FTimer.Free;
    if Assigned(FOnAfter) then
      FOnAfter(self);
  except
  end;
end;
{**************************************************}
constructor TJvNagScreen.Create(AOwner: TComponent);
begin
  inherited;

  FPicture:=TBitmap.Create;
  FTime:=5000;
  FDisappear:=true;
end;
{**************************************************}
procedure TJvNagScreen.Creer(Sender: TObject);
begin
  if (FPicture.Width<>0)and(FPicture.height<>0) then
  begin
    FForm:=TForm.Create(Application);
    FForm.Visible:=false;
    FForm.Borderstyle:=bsNone;
    FForm.Position:=poScreenCenter;
    FForm.caption:='';
    FForm.Width:=FPicture.Width;
    FForm.Height:=FPicture.Height;
    Fpict:=TImage.Create(FForm);
    FPict.Align:=alClient;
    FPict.AutoSize:=true;
    FPict.Picture.bitmap.assign(FPicture);
    Fpict.parent:=FFOrm;
    FPict.visible:=true;
    FPict.width:=Fpicture.width;
    FPict.height:=FPicture.height;
    FPict.left:=0;
    FPict.top:=0;

    if FDisappear then
       FPict.OnClick:=Cacher;

    FTimer:=TTimer.Create(self);
    FTimer.Interval:=FTime;
    FTimer.OnTimer:=Cacher;
    FTimer.Enabled:=True;

    if Assigned(FOnBefore) then
      FOnBefore(self);

    FForm.Showmodal;
    FForm.Free;
  end;
end;
{**************************************************}
destructor TJvNagScreen.Destroy;
begin
  FPicture.free;
  inherited;
end;
{**************************************************}
procedure TJvNagScreen.Execute;
begin
  Creer(self);
end;
{**************************************************}
procedure TJvNagScreen.Loaded;
begin
  if not(csDesigning in ComponentState) then
    Execute;
end;
{**************************************************}
procedure TJvNagScreen.SetPicture(const Value: TBitmap);
begin
  FPicture.Assign(Value);
end;
{**************************************************}
end.
