{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvScrollingLabel.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvScrollingLabel;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, 
  JvTypes, StdCtrls, ExtCtrls;

type
  TJvScrollingLabel = class(TCustomLabel)
  private
    Ftimer:TTimer;
    FOnMouseEnter : TNotifyEvent;
    FOnMouseLeave : TNotifyEvent;
    FOnCtl3DChanged : TNotifyEvent;
    FOnParentColorChanged : TNotifyEvent;
    FInterval:cardinal;
    Ftext:string;
    FScrolling:boolean;
    FScrollText:string;
    FNoGrap:boolean;
    FDirection:TLabelDirection;
    procedure FSetInterval(Value:cardinal);
    procedure SetScrolling(Value:boolean);
    procedure FSetScrollText(Value:string);
    procedure MouseEnter(var msg:Tmessage); message CM_MOUSEENTER;
    procedure MouseLeave(var msg:Tmessage); message CM_MOUSELEAVE;
    procedure CMCtl3DChanged(var Message: TMessage); message CM_CTL3DCHANGED;
    procedure CMParentColorChanged(var Message: TMessage); message CM_ParentColorChangeD;
    procedure Scroll(Sender: TObject);
    procedure SetGrap(Value:boolean);
  protected
    procedure Resize; override;
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property Alignment;
    property AutoSize;
    property FocusControl;
    property DragCursor;
    property DragMode;
    property Font;
    property ParentColor;
    property Color;
    property ShowHint;
    property ParentFont;
    property ParentShowHint;
    property Layout;
    property ShowAccelChar;
    property PopupMenu;
    property Transparent;
    property WordWrap;
    property Align;
    property Left;
    property Visible;
    property Top;
    property Height;
    property Width;
    property Cursor;
    property Enabled;
    property Hint;
    property ScrollInterval:cardinal read FInterval write FSetInterval default 50;
    property Scrolling : boolean read FScrolling write SetScrolling default true;
    property NoGrap:boolean read FNoGrap write SetGrap default false;
    property Text:string read FScrollText write FSetScrollText;
    property ScrollDirection:TLabelDirection read FDirection write FDirection default sdLeftToRight;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseMove;
    property OnMouseDown;
    property OnMouseUp;
    property OnStartDrag;
    property OnMouseEnter : TNotifyEvent read FonMouseEnter write FonMouseEnter;
    property OnMouseLeave : TNotifyEvent read FonMouseLeave write FonMouseLeave;
    property OnCtl3DChanged : TNotifyEvent read FonCtl3DChanged write FonCtl3DChanged;
    property OnParentColorChange : TNotifyEvent read FonParentColorChanged write FonParentColorChanged;
  end;

implementation

resourcestring
   RC_UrlSite                   =       'http://www.buypin.com';

{**************************************************}
constructor TJvScrollingLabel.Create(AOwner: TComponent);
begin
   FInterval:=50;
   FScrolling:=true;
   FNoGrap:=false;
   FScrollText:=RC_UrlSite;
   FDirection:=sdLeftToRight;
   inherited;
   ftimer:=TTimer.Create(self);
   ftimer.OnTimer:=Scroll;
   ftimer.Interval:=FInterval;
   ftimer.Enabled:=FScrolling;
   ftext:=RC_UrlSite;
   Caption:=FScrollText;
   AutoSize:=false;
end;
{**************************************************}
destructor TJvScrollingLabel.Destroy;
begin
   FTimer.free;
   inherited;
end;
{**************************************************}
procedure TJvScrollingLabel.CMCtl3DChanged(var Message: TMessage);
begin
   inherited;
   if Assigned(FonCtl3DChanged) then FonCtl3DChanged(self);
end;
{**************************************************}
procedure TJvScrollingLabel.CMParentColorChanged(var Message: TMessage);
begin
   inherited;
   if Assigned(FonParentColorChanged) then FonParentColorChanged(Self);
end;
{**************************************************}
procedure TJvScrollingLabel.MouseEnter(var msg:Tmessage);
begin
   inherited;
   if Assigned(FonMouseEnter) then FonMouseEnter(self);
end;
{**************************************************}
procedure TJvScrollingLabel.MouseLeave(var msg:Tmessage);
begin
   inherited;
   if Assigned(FonMouseLeave) then FonMouseLeave(self);
end;
{**************************************************}
procedure  TJvScrollingLabel.FSetInterval(Value:cardinal);
begin
   FTimer.Interval:=Value;
   FInterval:=Value;
end;
{**************************************************}
procedure TJvScrollingLabel.Resize;
begin
   inherited;
   SetGrap(FNOGrap);
end;
{**************************************************}
procedure TJvScrollingLabel.SetScrolling(Value:boolean);
begin
   if not(csDesigning in componentState) then
      FTimer.Enabled:=Value;
   FScrolling:=Value;
end;
{**************************************************}
procedure TJvScrollingLabel.Scroll(Sender: TObject);
begin
   if csDesigning in componentstate then
      exit;
   if (length(ftext)<>0) then
   begin
        if FDirection=sdLeftToRight then ftext:=ftext[length(ftext)]+Copy(ftext,1,length(ftext)-1)
        else ftext:=Copy(ftext,2,length(ftext)-1)+ftext[1];
        Caption:=ftext;
   end;
end;
{**************************************************}
procedure TJvScrollingLabel.FSetScrollText(Value:string);
begin
   if pos(#10,Value)<>0 then Value:=#10+Value;
   Caption:=Value;
   FScrolltext:=Value;
   Ftext:=Value;
   SetGrap(FNoGrap);
end;
{**************************************************}
procedure TJvScrollingLabel.SetGrap(Value:boolean);
var
   i,j:Integer;
begin
   FNoGrap:=Value;
   if FNoGrap then
      FText:=FScrolltext+' '
   else
   begin
      FText:=FScrollText;
      with TCanvas.Create do
      begin
         Handle:=GetDc(0);
         Font.Assign(self.Font);
         j:=0;
         i:=0;
         while (j<self.width)and (i<10000) do
         begin
            FText:=FText+' ';
            j:=TextWidth(FText);
            inc(i);
         end;
         ReleaseDc(0,handle);
      end;
   end;
end;
{**************************************************}
end.
