{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvSlider.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvSlider;

{$ObjExportAll On} 

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  extctrls;

type
  TJvSlider = class(TCustomControl)
  private
    FRuler:TBitmap;
    FThumb:TBitmap;
    FThumb1,FThumb2:TBitmap;
    FHor:boolean;
    FClicked:boolean;
    FTracking:boolean;
    FMaximum:Integer;
    FDifference:real;
    FPosition:Integer;
    FFrom:Integer;
    Fchanged:boolean;
    FChanging:boolean;
    FOnChanged,FOnStop,FOnBegin:TNotifyEvent;
    FAutoSize: boolean;
    FTimer:TTimer;
    procedure SetThumb(Value:TBitmap);
    procedure SetFRuler(Value:TBitmap);
    procedure ThumbChanged(Sender: TObject);
    procedure SetMaximum(Value:integer);
    procedure Calcule;
    procedure ReCalcule(Sender: TObject);
    procedure SetPosition(Value:integer);
    procedure SetAutoSize(const Value: boolean);
    procedure Loading(Sender: TObject);
  protected
  public
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;X, Y: integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;X, Y: integer); override;
    procedure Paint;override;
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property ImageRuler:TBitmap read FRuler write SetFRuler;
    property ImageThumb:TBitmap read FThumb write SetThumb;
    property Align;
    property Visible;
    property Enabled;
    property Cursor;
    property DragMode;
    property DragCursor;
    property ParentShowHint;
    property ShowHint;
    property TabOrder;
    property AutoSize:boolean read FAutoSize write SetAutoSize default true;
    property Horizontal:boolean read FHor write FHor default true;
    property Maximum:Integer read FMaximum write SetMaximum default 100;
    property Position:Integer read FPosition write SetPosition default 0;
    property OnClick;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyUp;
    property OnKeyPress;
    property OnDragOver;
    property OnDragDrop;
    property OnEndDrag;
    property OnStartDrag;
    property OnBeginChange:TnotifyEvent read FOnbegin write FOnBegin;
    property OnChanged:TnotifyEvent read FOnChanged write FOnChanged;
    property OnStopChanged:TNotifyEvent read FOnStop write FOnStop;
  end;

implementation

{$R RES_Slider.res}

{***************************************************}
procedure TJvSlider.Paint;
var
   t:Trect;
begin
   T.Left:=0;
   T.Top:=0;
   T.Right:=Width;
   T.Bottom:=Height;
   Canvas.StretchDraw(T,FRuler);
   if Fhor then
   begin
      //horizontal
      T.left:=round(FDifference*FPosition);
      if Height-Fthumb1.height<0 then
         T.top:=0
      else
          T.Top:=(height - FThumb1.Height) div 2;
      FFrom:=T.top;
      Canvas.Draw(T.left,T.top,FThumb1);
      Fclicked:=false;
   end
   else
   begin
      //vertical
      if Width-FThumb1.width<0 then
         T.left:=0
      else
          T.left:=(Width-Fthumb1.width) div 2;
      T.top:=round(FDifference*FPosition);
      FFrom:=T.left;
      Canvas.Draw(T.left,T.top,FThumb1);
      Fclicked:=false;
   end;
end;
{******************************************************}
procedure TJvSlider.SetMaximum(Value:integer);
begin
   FMaximum:=Value;
   if Fposition>FMaximum then FPosition:=FMaximum;
   Calcule;
   SetPosition(FPosition);
end;
{******************************************************}
procedure TJvSlider.SetPosition(Value:integer);
begin
   if Value>FMaximum then Value:=FMaximum;
   if Fposition<0 then FPosition:=0;
   Fposition:=Value;

   if not(FTracking) then
   begin
      Calcule;
      Paint;
   end;
end;
{******************************************************}
procedure TJvSlider.Calcule;
begin
   //calcul the difference between pixel
   if FHor then
      FDifference:=(Width-Fthumb1.width) / FMaximum
   else
      FDifference:=(height-Fthumb1.height) / FMaximum;
end;
{******************************************************}
procedure TJvSlider.ReCalcule(Sender: TObject);
begin
   Calcule;
end;
{******************************************************}
constructor TJvSlider.Create(AOwner: TComponent);
begin
   width:=191;
   height:=11;
   FRuler:=TBitmap.Create;
   FThumb:=TBitmap.Create;
   FThumb1:=TBitmap.Create;
   FThumb2:=TBitmap.Create;
   FClicked:=false;
   FMaximum:=100;
   FTracking:=false;
   FPosition:=0;
   FFrom:=0;
   Fchanged:=false;
   FHor:=true;
   FChanging:=false;
   inherited;
   FThumb.LoadFromResourceName(hInstance,'THUMB');
   FRuler.LoadFromResourceName(hInstance,'RULER');
   Calcule;
   FThumb.OnChange:=ThumbChanged;
   Self.OnResize:=ReCalcule;
   Calcule;
   FTimer:=TTimer.Create(self);
   FTImer.interval:=10;
   FTimer.ontimer:=Loading;
   FTimer.enabled:=true;
end;
{******************************************************}
destructor TJvSlider.Destroy;
begin
   Fruler.free;
   FThumb.free;
   FThumb1.free;
   FThumb2.free;
   inherited;
end;
{******************************************************}
procedure TJvSlider.MouseMove(Shift: TShiftState; X, Y: integer);
var
   i:Integer;
begin
   if (Ftracking) and (ssLeft in shift) then
   begin
      if FHor then
      begin
         if (y in [0..FThumb1.height])or(FChanging) then
         begin
            i:=x-Fthumb1.width div 2;
            if i>0 then
            begin
               Fchanging:=true;
               i:=round(i / FDifference);
               if i>FMaximum then i:=FMaximum;
               FPosition:=i;
               if Assigned(FOnChanged) then FOnChanged(self);
               paint;
            end;
         end;
      end
      else
      begin
         if (x in [0..FThumb1.width]) or (FChanging) then
         begin
            i:=y-Fthumb1.height div 2;
            if i>0 then
            begin
               Fchanging:=true;
               i:=round(i / FDifference);
               if i>Fmaximum then i:=FMaximum;
               FPosition:=i;
               if Assigned(FOnChanged) then FOnChanged(self);
               paint;
            end;
         end;
      end;
      paint;
   end;
end;
{******************************************************}
procedure TJvSlider.MouseDown(Button: TMouseButton; Shift: TShiftState;X, Y: integer);
var
   tmp:Tbitmap;
begin
   FTracking:=true;
   if Assigned(FOnBegin) then FOnBegin(Self);
   if not(Fchanged) then
   begin
      tmp:=TBitmap.Create;
      tmp.assign(Fthumb1);
      Fthumb1.Assign(Fthumb2);
      Fthumb2.assign(tmp);
      tmp.free;
      Fchanged:=true;
   end;
   paint;
end;
{******************************************************}
procedure TJvSlider.MouseUp(Button: TMouseButton; Shift: TShiftState;X, Y: integer);
var
   tmp:TBitmap;
begin
   FTracking:=false;
   FChanging:=false;
   if Fchanged then
   begin
      tmp:=TBitmap.Create;
      tmp.Assign(Fthumb1);
      Fthumb1.assign(FThumb2);
      Fthumb2.assign(tmp);
      tmp.free;
      Fchanged:=false;
   end;
   paint;
   if Assigned(FOnStop) then FOnStop(self);
end;
{******************************************************}
procedure TJvSlider.ThumbChanged(Sender: TObject);
var
   Src,dest:TRect;
begin
   dest.Left:=0;
   dest.Top:=0;
   dest.Right:=Fthumb.Width div 2;
   Dest.Bottom:=FThumb.height;
   FThumb1.Width:=dest.Right;
   FThumb1.Height:=Dest.Bottom;
   FThumb1.Canvas.CopyRect(dest,FThumb.canvas,dest);
   FThumb2.Width:=dest.Right;
   FThumb2.Height:=Dest.Bottom;
   dest.Left:=dest.Right;
   dest.top:=0;
   dest.bottom:=FThumb.height;
   dest.Right:=FThumb.width;
   src.left:=0;
   src.top:=0;
   src.right:=dest.left;
   src.bottom:=Fthumb.height;
   FThumb2.Canvas.CopyRect(src,FThumb.canvas,dest);
   Paint;
   Calcule;
end;
{******************************************************}
procedure TJvSlider.SetThumb(Value:TBitmap);
begin
   FThumb.Assign(Value);
   ThumbChanged(nil);
end;
{******************************************************}
procedure TJvSlider.SetFRuler(Value:TBitmap);
begin
   FRuler.Assign(Value);
   if (Value.width>0) and (Value.Height>0)and(FAutoSize) then
   begin
      self.height:=Value.height;
      self.width:=Value.width;
   end;
   Paint;
   Calcule;
end;
{******************************************************}
procedure TJvSlider.SetAutoSize(const Value: boolean);
begin
   FAutoSize := Value;
   if Value then
   begin
      if (FRuler.width>0) and (FRuler.Height>0) then
      begin
         self.height:=FRuler.height;
         self.width:=FRuler.width;
      end;
   end;
end;
{******************************************************}
procedure TJvSlider.Loading(Sender: TObject);
begin
   FTimer.enabled:=false;
   SetThumb(FThumb);
   ThumbChanged(self);
   Calcule;
   FTimer.free;
end;
{******************************************************}
end.
