{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvVisualId3v2.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvVisualId3v2;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, extctrls,
  Dialogs, JPEG, JvId3v2, comctrls, stdctrls, Menus, extdlgs,
  ShellApi;

type
  TJvPInformation = class(TPersistent)
  private
    function GetExperimental: string;
    function GetExtended: string;
    function GetFileName: string;
    function GetPresent: string;
    function GetSize: string;
    function GetSynchro: string;
    procedure SetExperimental(const Value: string);
    procedure SetExtended(const Value: string);
    procedure SetFileName(const Value: string);
    procedure SetPresent(const Value: string);
    procedure SetSize(const Value: string);
    procedure SetSynchro(const Value: string);
    function GetTagVersion: string;
    procedure SetTagVersion(const Value: string);
  protected
    FParent:TObject;
  public
  published
    property FileName:string read GetFileName write SetFileName;
    property TagPresent:string read GetPresent write SetPresent;
    property TagSize:string read GetSize write SetSize;
    property Unsynchro:string read GetSynchro write SetSynchro;
    property Experimental:string read GetExperimental write SetExperimental;
    property Extended:string read GetExtended write SetExtended;
    property TagVersion:string read GetTagVersion write SetTagVersion;
  end;

  TJvPMisc = class(TPersistent)
  private
    function GetCounter: string;
    function GetInvolved: TstringList;
    function GetPrice: string;
    function GetPurchased: string;
    function GetRating: string;
    function GetSeller: string;
    function GetUserEmail: string;
    procedure SetCounter(const Value: string);
    procedure SetInvolved(const Value: TstringList);
    procedure SetPrice(const Value: string);
    procedure SetPurchased(const Value: string);
    procedure SetRating(const Value: string);
    procedure SetSeller(const Value: string);
    procedure SetUserEmail(const Value: string);
  protected
  public
    FParent:TObject;
  published
    property InvolvedPersons:TstringList read GetInvolved write SetInvolved;
    property Seller:string read GetSeller write SetSeller;
    property Price:string read GetPrice write SetPrice;
    property PurchasedThe:string read GetPurchased write SetPurchased;
    property Counter:string read GetCounter write SetCounter;
    property Rating:string read GetRating write SetRating;
    property UserEmail:string read GetUserEmail write SetUserEmail;
  end;

  TJvPText1 = class(TPersistent)
  private
    function GetArtists: TstringList;
    function GetBand: string;
    function GetConductor: string;
    function GetCreation: string;
    function GetLanguages: TstringList;
    function GetLyricists: TstringList;
    function GetRemixed: string;
    function GetSubTitle: string;
    function GetTitle: string;
    procedure SetArtists(const Value: TstringList);
    procedure SetBand(const Value: string);
    procedure SetConductor(const Value: string);
    procedure SetCreation(const Value: string);
    procedure SetLanguages(const Value: TstringList);
    procedure SetLyricists(const Value: TstringList);
    procedure SetRemixed(const Value: string);
    procedure SetSubTitle(const Value: string);
    procedure SetTitle(const Value: string);
    function GetComposers: TstringList;
    procedure SetComposers(const Value: TstringList);
  protected
  public
    FParent:TObject;
  published
    property Creation:string read GetCreation write SetCreation;
    property Title:string read GetTitle write SetTitle;
    property SubTitle:string read GetSubTitle write SetSubTitle;
    property Artists:TstringList read GetArtists write SetArtists;
    property Band:string read GetBand write SetBand;
    property Conductor:string read GetConductor write SetConductor;
    property RemixedBy:string read GetRemixed write SetRemixed;
    property Composers:TstringList read GetComposers write SetComposers;
    property Lyricists:TstringList read GetLyricists write SetLyricists;
    property Languages:TstringList read GetLanguages write SetLanguages;
  end;

  TJvPText2 = class(TPersistent)
  private
    function GetAlbum: string;
    function GetContentType: string;
    function GeTDate: string;
    function GetISRC: string;
    function GetMediaType: string;
    function GetPart: string;
    function GetRecorded: string;
    function GeTTime: string;
    function GetTrack: string;
    function GetYear: string;
    procedure SetAlbum(const Value: string);
    procedure SetContentType(const Value: string);
    procedure SeTDate(const Value: string);
    procedure SetISRC(const Value: string);
    procedure SetMediaType(const Value: string);
    procedure SetPart(const Value: string);
    procedure SetRecorded(const Value: string);
    procedure SeTTime(const Value: string);
    procedure SetYear(const Value: string);
    procedure SetTrack(const Value: string);
  protected
  public
    FParent:TObject;
  published
    property ContentType:string read GetContentType write SetContentType;
    property Album:string read GetAlbum write SetAlbum;
    property Part:string read GetPart write SetPart;
    property Track:string read GetTrack write SetTrack;
    property ISRC:string read GetISRC write SetISRC;
    property Year:string read GetYear write SetYear;
    property Date:string read GeTDate write SeTDate;
    property Time:string read GeTTime write SeTTime;
    property recordedAt:string read GetRecorded write SetRecorded;
    property MediaType:string read GetMediaType write SetMediaType;
  end;

  TJvPText3 = class(TPersistent)
  private
    function GetBPM: string;
    function GetCopyright: string;
    function GetDelay: string;
    function GetEncodedBy: string;
    function GetEncoder: string;
    function GetInitialKey: string;
    function GetLength: string;
    function GetOFileName: string;
    function GetPublisher: string;
    function GetSize: string;
    procedure SetBPM(const Value: string);
    procedure SetCopyright(const Value: string);
    procedure SetDelay(const Value: string);
    procedure SetEncodedBy(const Value: string);
    procedure SetEncoder(const Value: string);
    procedure SetInitialKey(const Value: string);
    procedure SetLength(const Value: string);
    procedure SetOFileName(const Value: string);
    procedure SetPublisher(const Value: string);
    procedure SetSize(const Value: string);
  protected
  public
    FParent:TObject;
  published
    property BPM:string read GetBPM write SetBPM;
    property Copyright:string read GetCopyright write SetCopyright;
    property Publisher:string read GetPublisher write SetPublisher;
    property EncodedBy:string read GetEncodedBy write SetEncodedBy;
    property Encoder:string read GetEncoder write SetEncoder;
    property OriginalFileName:string read GetOFileName write SetOFileName;
    property Length:string read GetLength write SetLength;
    property Size:string read GetSize write SetSize;
    property Delay:string read GetDelay write SetDelay;
    property InitialKey:string read GetInitialKey write SetInitialKey;
  end;

  TJvPText4 = class(TPersistent)
  private
    function GetFileOwner: string;
    function GetFileType: string;
    function GetOAlbum: string;
    function GetOArtists: TstringList;
    function GetOLyricists: TstringList;
    function GetRadioName: string;
    function GetRadioOwner: string;
    function GetUserDefined: TstringList;
    function GetYear: string;
    procedure SetFileOwner(const Value: string);
    procedure SetFileType(const Value: string);
    procedure SetOAlbum(const Value: string);
    procedure SetOArtists(const Value: TstringList);
    procedure SetOLyricists(const Value: TstringList);
    procedure SetRadioName(const Value: string);
    procedure SetRadioOwner(const Value: string);
    procedure SetUserDefined(const Value: TstringList);
    procedure SetYear(const Value: string);
  protected
  public
    FParent:TObject;
  published
    property OriginalAlbum:string read GetOAlbum write SetOAlbum;
    property OriginalArtists:TstringList read GetOArtists write SetOArtists;
    property OriginalLyricists:TstringList read GetOLyricists write SetOLyricists;
    property OriginalYear:string read GetYear write SetYear;
    property FileOwner:string read GetFileOwner write SetFileOwner;
    property FileType:string read GetFileType write SetFileType;
    property RadioName:string read GetRadioName write SetRadioName;
    property RadioOwner:string read GetRadioOwner write SetRadioOwner;
    property UserDefined:TstringList read GetUserDefined write SetUserDefined;
  end;

  TJvPTexts = class(TPersistent)
  private
    FJvText1: TJvPText1;
    FJvText2: TJvPText2;
    FJvText3: TJvPText3;
    FJvText4: TJvPText4;
  protected
  public
    constructor Create;
    destructor Destroy;override;
  published
    property Text1:TJvPText1 read FJvText1 write FJvText1;
    property Text2:TJvPText2 read FJvText2 write FJvText2;
    property Text3:TJvPText3 read FJvText3 write FJvText3;
    property Text4:TJvPText4 read FJvText4 write FJvText4;
  end;

  TJvPUrls = class(TPersistent)
  private
    function GetArtistPage: string;
    function GetAudioFile: string;
    function GetAudioSource: string;
    function GetCommercial: string;
    function GetCopyright: string;
    function GetPayment: string;
    function GetPublishers: string;
    function GetRadioStation: string;
    function GetUserDefined: TstringList;
    procedure SetArtistPage(const Value: string);
    procedure SetAudioFile(const Value: string);
    procedure SetAudioSource(const Value: string);
    procedure SetCommercial(const Value: string);
    procedure SetCopyright(const Value: string);
    procedure SetPayment(const Value: string);
    procedure SetPublishers(const Value: string);
    procedure SetRadioStation(const Value: string);
    procedure SetUserDefined(const Value: TstringList);
  protected
    FParent:TObject;
  public
  published
    property UserDefined:TstringList read GetUserDefined write SetUserDefined;
    property Commercial:string read GetCommercial write SetCommercial;
    property Copyright:string read GetCopyright write SetCopyright;
    property AudioFile:string read GetAudioFile write SetAudioFile;
    property ArtistPage:string read GetArtistPage write SetArtistPage;
    property AudioSource:string read GetAudioSource write SetAudioSource;
    property RadioStation:string read GetRadioStation write SetRadioStation;
    property Payment:string read GetPayment write SetPayment;
    property Publishers:string read GetPublishers write SetPublishers;
  end;

  TJvPId3v2Editor = class(TPersistent)
  private
    FInformations: TJvPInformation;
    FMisc: TJvPMisc;
    FTexts: TJvPTexts;
    FUrls: TJvPUrls;
  protected
  public
    constructor Create;
    destructor Destroy;override;
  published
    property Informations:TJvPInformation read FInformations write FInformations;
    property Misc:TJvPMisc read FMisc write FMisc;
    property Texts:TJvPTexts read FTexts write FTexts;
    property Urls:TJvPUrls read FUrls write FUrls;
  end;

  TJvPId3v2TreeView = class(TPersistent)
  private
    function GetHideSelection: boolean;
    function GetHotTrack: boolean;
    function GetReadOnly: boolean;
    function GetWidth: Integer;
    procedure SetHideSelection(const Value: boolean);
    procedure SetHotTrack(const Value: boolean);
    procedure SetReadOnly(const Value: boolean);
    procedure SetWidth(const Value: integer);
  protected
  public
    FParent:TObject;
  published
    property HotTrack:boolean read GetHotTrack write SetHotTrack;
    property HideSelection:boolean read GetHideSelection write SetHideSelection;
    property Width:Integer read GetWidth write SetWidth;
    property ReadOnly:boolean read GetReadOnly write SetReadOnly;
  end;

  TJvVisualId3v2 = class(TWinControl)
  private
    FId3v2:TJvId3v2;
    FPhotosTypes:TstringList;
    FDatas: TJvPId3v2Editor;
    FTreeEdit: TJvPId3v2TreeView;

    //Controls
    FTreeView:TTreeView;
    FPageControl:TPageControl;
    FImagePopup:TPopupMenu;
    FPhotoImg:TImage;
    FPhotoTypes, FUserD, FInvolved: TCombobox;
    FArtists, FLyricists, FLanguages:TCombobox;
    FUserTexts, FOArtists, FOLyricists, FPublishersLst:TCombobox;

    //Labels
    FFileName, FTagSize, FTagVersion, FUnsynchro, FExperimental,
    FExtended, FPresent:TLabel;
    FCommercial, FCopyright, FAudioFile, FArtistPage, FAudioSource,
    FRadioStation, FPayment, FPublishers, FSeller, FPrice, FDatePur: TLabel;
    FCreation, FTitle, FSubTitle, FBand, FConductor, FRemixed: TLabel;
    FPhotoDesc: TStaticText;
    FContent, FAlbum, FPart, FTrack, FISRC, FYear, FDate, FTime, FRecordedAt,
    FMediaType:TLabel;
    FBPM, FCopyrightTxt, FPublisher, FEncoded, FEncoder, FOFileName,
    FLength, FSize, FDelay, FInitialKey: TLabel;
    FOAlbum, FOYear, FFileOwner, FFileType,
    FRadioName, FRadioOwner:TLabel;
    FCounter, FRating, FUserMail: TLabel;

    //TabSheets
    FInfoSheet, FUrlSheet, FPhotoSheet, FMiscSheet, FText1Sheet,
    FText2Sheet, FText3Sheet, FText4Sheet: TTabSheet;

    procedure CreateControls;
    procedure CreateMenu;
    procedure CreatePages;
    procedure TreeChange(Sender: TObject; Node: TTreeNode);
    function GetFileName: TFileName;
    procedure SetFileName(const Value: TFileName);
    procedure SaveImage(Sender: TObject);
    procedure StretchImage(Sender: TObject);
    procedure PhotoChange(Sender: TObject);
    procedure UrlClick(Sender: TObject);
    procedure UrlCombo(Sender: TObject);
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;

  published
    property FileName:TFileName read GetFileName write SetFileName;
    property Datas:TJvPId3v2Editor read FDatas write FDatas;
    property TreeView:TJvPId3v2TreeView read FTreeEdit write FTreeEdit;
    property Align;
    property OnDockDrop;
    property OnDockOver;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnUnDock;
    property TabStop;
    property TabOrder;
    property Enabled;
    property ShowHint;
    property Visible;
  end;

implementation

///////////////////////////////////////////////////////////
// TJvVisualId3v2
///////////////////////////////////////////////////////////

{******************************************************************************}
constructor TJvVisualId3v2.Create(AOwner: TComponent);
begin
   inherited;
   self.Width:=430;
   self.Height:=300;
   self.Parent:=TWinControl(AOwner);

   FId3v2:=TJvId3v2.Create(self);

   CreateControls;
   CreateMenu;
   CreatePages;

   FPhotosTypes:=TstringList.Create;

   FDatas:=TJvPId3v2Editor.Create;
   FDatas.FInformations.FParent:=self;
   FDatas.FMisc.FParent:=self;
   FDatas.FUrls.FParent:=self;
   FDatas.FTexts.FJvText1.FParent:=self;
   FDatas.FTexts.FJvText2.FParent:=self;
   FDatas.FTexts.FJvText3.FParent:=self;
   FDatas.FTexts.FJvText4.FParent:=self;

   FTreeEdit:=TJvPId3v2TreeView.Create;
   FTreeEdit.FParent:=self;
end;
{******************************************************************************}
procedure TJvVisualId3v2.CreateControls;
var
 it:TMenuItem;
begin
   FTreeView:=TTreeView.Create(self);
   FTreeView.Parent:=self;
   FTreeView.Align:=alLeft;
   FTreeView.Width:=120;
   FTreeView.ReadOnly:=true;
   FTreeView.HotTrack:=true;
   FTreeView.HideSelection:=false;

   FPageControl:=TPageControl.Create(self);
   FPageControl.Parent:=self;
   FPageControl.Align:=alClient;

   FImagePopup:=TPopupMenu.Create(self);

   it:=TMenuItem.Create(FImagePopup);
   it.Caption:='&Save...';
   it.OnClick:=SaveImage;
   FImagePopup.Items.Add(it);

   it:=TMenuItem.Create(FImagePopup);
   it.Caption:='Stretch';
   it.OnClick:=StretchImage;
   FImagePopup.Items.Add(it);
end;
{******************************************************************************}
procedure TJvVisualId3v2.CreateMenu;
var
  Node:TTreeNode;
begin
   Node:=FTreeView.Items.AddChild(nil,'Informations');
   Node.Selected:=true;
   FTreeView.Items.AddChild(nil,'Misc');
   FTreeView.Items.AddChild(nil,'Photos');
   Node:=FTreeView.Items.AddChild(nil,'Texts');
   FTreeView.Items.AddChild(Node,'Part 1');
   FTreeView.Items.AddChild(Node,'Part 2');
   FTreeView.Items.AddChild(Node,'Part 3');
   FTreeView.Items.AddChild(Node,'Part 4');
   FTreeView.Items.AddChild(nil,'Urls');
   FTreeView.OnChange:=TreeChange;
end;
{******************************************************************************}
procedure TJvVisualId3v2.CreatePages;

  procedure CreateGeneralSheet(var Sheet:TTabSheet;Caption:string);
  begin
     Sheet:=TTabSheet.Create(FPageControl);
     Sheet.Parent:=FPageControl;
     Sheet.PageControl:=FPageControl;
     Sheet.Caption:=Caption;
     Sheet.TabVisible:=false;
  end;

  /////////////////////////////////////////////////////////
  ////Info Sheet
  /////////////////////////////////////////////////////////

  procedure CreateInfoSheet;
  var
    FPanel:TPanel;

    procedure CreateLabel1(vTop:Integer;vCaption:string);
    begin
       with TLabel.Create(FInfoSheet) do
       begin
          Parent:=FInfoSheet;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;
    
    function CreateLabel2(vTop:Integer;vCaption:string):TLabel;
    begin
       result:=TLabel.Create(FPanel);
       with result do
       begin
          Parent:=FPanel;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;

  begin
     CreateGeneralSheet(FInfoSheet,'Informations');

     FPanel:=TPanel.Create(FInfoSheet);
     with FPanel do
     begin
        Caption:='';
        Left:=80;Top:=10;Width:=210;Height:=165;
        Parent:=FInfoSheet;
        BevelOuter:=bvRaised;
     end;

     CreateLabel1(15,'FileName');
     CreateLabel1(39,'Tag Present');
     CreateLabel1(63,'Tag Size');
     CreateLabel1(87,'Tag Version');
     CreateLabel1(111,'Unsynchro');
     CreateLabel1(135,'Experimental');
     CreateLabel1(159,'Extended');

     FFileName    := CreateLabel2(  4,'-');
     FPresent     := CreateLabel2( 28,'-');
     FTagSize     := CreateLabel2( 52,'-');
     FTagVersion  := CreateLabel2( 76,'-');
     FUnsynchro   := CreateLabel2(100,'-');
     FExperimental:= CreateLabel2(124,'-');
     FExtended    := CreateLabel2(149,'-');
  end;

  /////////////////////////////////////////////////////////
  ////Url Sheet
  /////////////////////////////////////////////////////////

  procedure CreateUrlSheet;
  var
    FPanel:TPanel;
    
    procedure CreateLabel1(vTop:Integer;vCaption:string);
    begin
       with TLabel.Create(FUrlSheet) do
       begin
          Parent:=FUrlSheet;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;
    
    function CreateLabel2(vTop:Integer;vCaption:string):TLabel;
    begin
       result:=TLabel.Create(FPanel);
       with result do
       begin
          Parent:=FPanel;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;

  begin
     CreateGeneralSheet(FUrlSheet,'Urls');

     FPanel:=TPanel.Create(FUrlSheet);
     with FPanel do
     begin
        Caption:='';
        Left:=80;Top:=10;Width:=210;Height:=225;
        Parent:=FUrlSheet;
        BevelOuter:=bvRaised;
     end;

     CreateLabel1(15,'Commercial');
     CreateLabel1(39,'Copyright');
     CreateLabel1(63,'Audio File');
     CreateLabel1(87,'Artist Page');
     CreateLabel1(111,'Audio Source');
     CreateLabel1(135,'Radio Station');
     CreateLabel1(159,'Payment');
     CreateLabel1(183,'Publishers');
     CreateLabel1(207,'User Defined');

     FCommercial  :=  CreateLabel2(  4,'-');
     FCopyright   :=  CreateLabel2( 28,'-');
     FAudioFile   :=  CreateLabel2( 52,'-');
     FArtistPage  :=  CreateLabel2( 76,'-');
     FAudioSource :=  CreateLabel2(100,'-');
     FRadioStation:=  CreateLabel2(124,'-');
     FPayment     :=  CreateLabel2(149,'-');
     FPublishers  :=  CreateLabel2(173,'-');

     FUserD:=TCombobox.Create(FPanel);
     with FUserD do
     begin
        Parent:=FPanel;
        Width:=FPanel.Width-Left-10;
        Left:=6;
        Top:=194;
        style:=csDropDownList;
        OnChange:=UrlCombo;
     end;
  end;

  /////////////////////////////////////////////////////////
  ////Photo Sheet
  /////////////////////////////////////////////////////////

  procedure CreatePhotoSheet;
  var
    FPanel:TPanel;
  begin
     FPhotoSheet:=TTabSheet.Create(FPageControl);
     FPhotoSheet.Parent:=FPageControl;
     FPhotoSheet.PageControl:=FPageControl;
     FPhotoSheet.Caption:='Photos';
     FPhotoSheet.TabVisible:=false;

     with TLabel.Create(FPhotoSheet) do
     begin
        Parent:=FPhotoSheet;
        AutoSize:=true;
        Left:=6;Top:=15;
        Caption:='Types';
     end;

     with TLabel.Create(FPhotoSheet) do
     begin
        Parent:=FPhotoSheet;
        AutoSize:=true;
        Left:=6;Top:=39;
        Caption:='Description';
     end;

     FPanel:=TPanel.Create(FPhotoSheet);
     with FPanel do
     begin
        Caption:='';
        Left:=80;Top:=10;Width:=210;Height:=60;
        Parent:=FPhotoSheet;
        BevelOuter:=bvRaised;
     end;

     FPhotoTypes:=TCombobox.Create(FPanel);
     with FPhotoTypes do
     begin
        Parent:=FPanel;
        Width:=FPanel.Width-Left-10;
        Left:=6;
        Top:=4;
        style:=csDropDownList;
        OnChange:=PhotoChange;
        OnClick:=PhotoChange;
        OnDropDown:=PhotoChange;
     end;

     FPhotoDesc:=TStaticText.Create(FPanel);
     with FPhotoDesc do
     begin
        Parent:=FPanel;
        AutoSize:=false;
        Left:=6;Top:=28;
        Width:=FPanel.Width-Left-10;
        Height:=28;
        Caption:='-';
     end;

     FPhotoImg:=TImage.Create(FPhotoSheet);
     with FPhotoImg do
     begin
        Parent:=FPhotoSheet;
        AutoSize:=false;
        Left:=10;Top:=85;
        Width:=FPhotoSheet.Width-20;
        Height:=FPhotoSheet.Height-95;
        Stretch:=false;
        Center:=true;
        PopupMenu:=FImagePopup;
     end;
  end;

  /////////////////////////////////////////////////////////
  ////Misc Sheet
  /////////////////////////////////////////////////////////

  procedure CreateMiscSheet;
  var
    FPanel:TPanel;

    procedure CreateLabel1(vTop:Integer;vCaption:string);
    begin
       with TLabel.Create(FMiscSheet) do
       begin
          Parent:=FMiscSheet;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;
    
    function CreateLabel2(vTop:Integer;vCaption:string):TLabel;
    begin
       result:=TLabel.Create(FPanel);
       with result do
       begin
          Parent:=FPanel;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;

  begin
     FMiscSheet:=TTabSheet.Create(FPageControl);
     FMiscSheet.Parent:=FPageControl;
     FMiscSheet.PageControl:=FPageControl;
     FMiscSheet.Caption:='Misc';
     FMiscSheet.TabVisible:=false;

     FPanel:=TPanel.Create(FMiscSheet);
     with FPanel do
     begin
        Caption:='';
        Left:=80;Top:=10;Width:=210;Height:=100;
        Parent:=FMiscSheet;
        BevelOuter:=bvRaised;
     end;

     CreateLabel1(15,'Involved Pers');
     CreateLabel1(39,'Seller');
     CreateLabel1(63,'Price');
     CreateLabel1(87,'Purchased the');

     FInvolved:=TCombobox.Create(FPanel);
     with FInvolved do
     begin
        Parent:=FPanel;
        Width:=FPanel.Width-Left-10;
        Left:=6;
        Top:=4;
        style:=csDropDownList;
     end;

     FSeller := CreateLabel2( 28,'-');
     FPrice  := CreateLabel2( 52,'-');
     FDatePur:= CreateLabel2( 76,'-');

     //Popularimeter, play counter
     CreateLabel1(121,'Counter');
     CreateLabel1(145,'Rating');
     CreateLabel1(169,'User Email');

     FPanel:=TPanel.Create(FMiscSheet);
     with FPanel do
     begin
        Caption:='';
        Left:=80;Top:=115;Width:=210;Height:=75;
        Parent:=FMiscSheet;
        BevelOuter:=bvRaised;
     end;

     FCounter   := CreateLabel2(6,'-');
     FRating    := CreateLabel2(30,'-');
     FUserMail  := CreateLabel2(54,'-');
  end;

  /////////////////////////////////////////////////////////
  ////Text1 Sheet
  /////////////////////////////////////////////////////////
  
  procedure CreateText1Sheet;
  var
    FPanel:TPanel;

    procedure CreateLabel1(vTop:Integer;vCaption:string);
    begin
       with TLabel.Create(FText1Sheet) do
       begin
          Parent:=FText1Sheet;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;
    
    function CreateLabel2(vTop:Integer;vCaption:string):TLabel;
    begin
       result:=TLabel.Create(FPanel);
       with result do
       begin
          Parent:=FPanel;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;

    function CreateCombo2(vTop:integer):TCombobox;
    begin
       result:=TCombobox.Create(FPanel);
       with result do
       begin
          Parent:=FPanel;
          Left:=6;Top:=vTop;Width:=FPanel.Width-12;
          style:=csDropDownList;
       end;
    end;

  begin
     FText1Sheet:=TTabSheet.Create(FPageControl);
     FText1Sheet.Parent:=FPageControl;
     FText1Sheet.PageControl:=FPageControl;
     FText1Sheet.Caption:='Part 1';
     FText1Sheet.TabVisible:=false;

     FPanel:=TPanel.Create(FText1Sheet);
     with FPanel do
     begin
        Caption:='';
        Left:=80;Top:=10;Width:=210;Height:=250;
        Parent:=FText1Sheet;
        BevelOuter:=bvRaised;
     end;

     CreateLabel1( 15,'Creation');
     CreateLabel1( 39,'Title');
     CreateLabel1( 63,'SubTitle');
     CreateLabel1( 87,'Artist(s)');
     CreateLabel1(111,'Band');
     CreateLabel1(135,'Conductor');
     CreateLabel1(159,'Remixed by');
     CreateLabel1(183,'Composer(s)');
     CreateLabel1(207,'Lyricist(s)');
     CreateLabel1(231,'Langague(s)');

     FCreation     :=  CreateLabel2(  4,'-');
     FTitle        :=  CreateLabel2( 28,'-');
     FSubTitle     :=  CreateLabel2( 52,'-');
     FArtists      :=  CreateCombo2( 74);
     FBand         :=  CreateLabel2(100,'-');
     FConductor    :=  CreateLabel2(124,'-');
     FRemixed      :=  CreateLabel2(149,'-');
     FPublishersLst:=  CreateCombo2(171);
     FLyricists    :=  CreateCombo2(195);
     FLanguages    :=  CreateCombo2(220);
  end;

  /////////////////////////////////////////////////////////
  ////Text2 Sheet
  /////////////////////////////////////////////////////////

  procedure CreateText2Sheet;
  var
    FPanel:TPanel;

    procedure CreateLabel1(vTop:Integer;vCaption:string);
    begin
       with TLabel.Create(FText2Sheet) do
       begin
          Parent:=FText2Sheet;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;
    
    function CreateLabel2(vTop:Integer;vCaption:string):TLabel;
    begin
       result:=TLabel.Create(FPanel);
       with result do
       begin
          Parent:=FPanel;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;

    function CreateCombo2(vTop:integer):TCombobox;
    begin
       result:=TCombobox.Create(FPanel);
       with result do
       begin
          Parent:=FPanel;
          Left:=6;Top:=vTop;Width:=FPanel.Width-12;
          style:=csDropDownList;
       end;
    end;

  begin
     FText2Sheet:=TTabSheet.Create(FPageControl);
     FText2Sheet.Parent:=FPageControl;
     FText2Sheet.PageControl:=FPageControl;
     FText2Sheet.Caption:='Part 2';
     FText2Sheet.TabVisible:=false;

     FPanel:=TPanel.Create(FText2Sheet);
     with FPanel do
     begin
        Caption:='';
        Left:=80;Top:=10;Width:=210;Height:=250;
        Parent:=FText2Sheet;
        BevelOuter:=bvRaised;
     end;

     CreateLabel1( 15,'Content Type');
     CreateLabel1( 39,'Album');
     CreateLabel1( 63,'Part');
     CreateLabel1( 87,'Track');
     CreateLabel1(111,'ISRC');
     CreateLabel1(135,'Year');
     CreateLabel1(159,'Date');
     CreateLabel1(183,'Time');
     CreateLabel1(207,'Recorded at');
     CreateLabel1(231,'Media Type');

     FContent      :=  CreateLabel2(  4,'-');
     FAlbum        :=  CreateLabel2( 28,'-');
     FPart         :=  CreateLabel2( 52,'-');
     FTrack        :=  CreateLabel2( 76,'-');
     FISRC         :=  CreateLabel2(100,'-');
     FYear         :=  CreateLabel2(124,'-');
     FDate         :=  CreateLabel2(149,'-');
     FTime         :=  CreateLabel2(173,'-');
     FRecordedAt   :=  CreateLabel2(197,'-');
     FMediaType    :=  CreateLabel2(221,'-');
  end;

  /////////////////////////////////////////////////////////
  ////Text3 Sheet
  /////////////////////////////////////////////////////////

  procedure CreateText3Sheet;
  var
    FPanel:TPanel;

    procedure CreateLabel1(vTop:Integer;vCaption:string);
    begin
       with TLabel.Create(FText3Sheet) do
       begin
          Parent:=FText3Sheet;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;

    function CreateLabel2(vTop:Integer;vCaption:string):TLabel;
    begin
       result:=TLabel.Create(FPanel);
       with result do
       begin
          Parent:=FPanel;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;

    function CreateCombo2(vTop:integer):TCombobox;
    begin
       result:=TCombobox.Create(FPanel);
       with result do
       begin
          Parent:=FPanel;
          Left:=6;Top:=vTop;Width:=FPanel.Width-12;
          style:=csDropDownList;
       end;
    end;

  begin
     FText3Sheet:=TTabSheet.Create(FPageControl);
     FText3Sheet.Parent:=FPageControl;
     FText3Sheet.PageControl:=FPageControl;
     FText3Sheet.Caption:='Part 3';
     FText3Sheet.TabVisible:=false;

     FPanel:=TPanel.Create(FText3Sheet);
     with FPanel do
     begin
        Caption:='';
        Left:=80;Top:=10;Width:=210;Height:=250;
        Parent:=FText3Sheet;
        BevelOuter:=bvRaised;
     end;

     CreateLabel1( 15,'BPM');
     CreateLabel1( 39,'Copyright');
     CreateLabel1( 63,'Publisher');
     CreateLabel1( 87,'Encoded By');
     CreateLabel1(111,'Encoder');
     CreateLabel1(135,'O. FileName');
     CreateLabel1(159,'Length');
     CreateLabel1(183,'Size');
     CreateLabel1(207,'Delay');
     CreateLabel1(231,'Initial Key');

     FBPM          :=  CreateLabel2(  4,'-');
     FCopyrightTxt :=  CreateLabel2( 28,'-');
     FPublisher    :=  CreateLabel2( 52,'-');
     FEncoded      :=  CreateLabel2( 76,'-');
     FEncoder      :=  CreateLabel2(100,'-');
     FOFileName    :=  CreateLabel2(124,'-');
     FLength       :=  CreateLabel2(149,'-');
     FSize         :=  CreateLabel2(173,'-');
     FDelay        :=  CreateLabel2(197,'-');
     FInitialKey   :=  CreateLabel2(221,'-');
  end;

  /////////////////////////////////////////////////////////
  ////Text4 Sheet
  /////////////////////////////////////////////////////////

  procedure CreateText4Sheet;
  var
    FPanel:TPanel;

    procedure CreateLabel1(vTop:Integer;vCaption:string);
    begin
       with TLabel.Create(FText4Sheet) do
       begin
          Parent:=FText4Sheet;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;

    function CreateLabel2(vTop:Integer;vCaption:string):TLabel;
    begin
       result:=TLabel.Create(FPanel);
       with result do
       begin
          Parent:=FPanel;
          AutoSize:=true;
          Left:=6;Top:=vTop;
          Caption:=vCaption;
       end;
    end;

    function CreateCombo2(vTop:integer):TCombobox;
    begin
       result:=TCombobox.Create(FPanel);
       with result do
       begin
          Parent:=FPanel;
          Left:=6;Top:=vTop;Width:=FPanel.Width-12;
          style:=csDropDownList;
       end;
    end;

  begin
     FText4Sheet:=TTabSheet.Create(FPageControl);
     FText4Sheet.Parent:=FPageControl;
     FText4Sheet.PageControl:=FPageControl;
     FText4Sheet.Caption:='Part 4';
     FText4Sheet.TabVisible:=false;

     FPanel:=TPanel.Create(FText4Sheet);
     with FPanel do
     begin
        Caption:='';
        Left:=80;Top:=10;Width:=210;Height:=225;
        Parent:=FText4Sheet;
        BevelOuter:=bvRaised;
     end;

     CreateLabel1( 15,'O. Album');
     CreateLabel1( 39,'O. Artist(s)');
     CreateLabel1( 63,'O. Lyricist(s)');
     CreateLabel1( 87,'O. Year');
     CreateLabel1(111,'File Owner');
     CreateLabel1(135,'File Type');
     CreateLabel1(159,'Radio Name');
     CreateLabel1(183,'Radio Owner');
     CreateLabel1(207,'User Defined');

     FOAlbum       :=  CreateLabel2(  4,'-');
     FOArtists     :=  CreateCombo2( 26);
     FOLyricists   :=  CreateCombo2( 50);
     FOYear        :=  CreateLabel2( 76,'-');
     FFileOwner    :=  CreateLabel2(100,'-');
     FFileType     :=  CreateLabel2(124,'-');
     FRadioName    :=  CreateLabel2(149,'-');
     FRadioOwner   :=  CreateLabel2(173,'-');
     FUserTexts    :=  CreateCombo2(194);
  end;

begin
   CreateInfoSheet;
   CreateUrlSheet;
   CreatePhotoSheet;
   CreateMiscSheet;
   CreateText1Sheet;
   CreateText2Sheet;
   CreateText3Sheet;
   CreateText4Sheet;

   FPageControl.ActivePage:=FInfoSheet;
end;
{******************************************************************************}
destructor TJvVisualId3v2.Destroy;
begin
   FTreeView.Free;
   FPageControl.Free;
   FId3v2.Free;
   FPhotosTypes.Free;
   FDatas.Free;
   FTreeEdit.Free;

   inherited;
end;
{******************************************************************************}
function TJvVisualId3v2.GetFileName: TFileName;
begin
   Result:=FId3v2.FileName;
end;
{******************************************************************************}
procedure TJvVisualId3v2.PhotoChange(Sender: TObject);
var
  i:Integer;

   procedure SetIt(Value:TPicture;Text:string);
   begin
      FPhotoImg.Picture.Assign(Value);
      FPhotoDesc.Caption:=Text;
   end;

begin
   if FPhotoTypes.ItemIndex=-1 then
     SetIt(nil,'')
   else
   begin
      i:=FPhotosTypes.IndexOf(FPhotoTypes.Items[FPhotoTypes.ItemIndex]);
      if FPhotoTypes.Tag=i then exit;
      FPhotoTypes.Tag:=i;
      case i of
        0  : SetIt(FId3v2.Images.Pictures.Artist,FId3v2.Images.Infos.Artist);
        1  : SetIt(FId3v2.Images.Pictures.Band,FId3v2.Images.Infos.Band);
        2  : SetIt(FId3v2.Images.Pictures.BandLogo,FId3v2.Images.Infos.BandLogo);
        3  : SetIt(FId3v2.Images.Pictures.ColouredFish,FId3v2.Images.Infos.ColouredFish);
        4  : SetIt(FId3v2.Images.Pictures.Composer,FId3v2.Images.Infos.Composer);
        5  : SetIt(FId3v2.Images.Pictures.Conductor,FId3v2.Images.Infos.Conductor);
        6  : SetIt(FId3v2.Images.Pictures.CoverBack,FId3v2.Images.Infos.CoverBack);
        7  : SetIt(FId3v2.Images.Pictures.CoverFront,FId3v2.Images.Infos.CoverFront);
        8  : SetIt(FId3v2.Images.Pictures.DuringPerformance,FId3v2.Images.Infos.DuringPerformance);
        9  : SetIt(FId3v2.Images.Pictures.DuringRecording,FId3v2.Images.Infos.DuringRecording);
        10 : SetIt(FId3v2.Images.Pictures.FileIcon,FId3v2.Images.Infos.FileIcon);
        11 : SetIt(FId3v2.Images.Pictures.Illustration,FId3v2.Images.Infos.Illustration);
        12 : SetIt(FId3v2.Images.Pictures.LeadArtist,FId3v2.Images.Infos.LeadArtist);
        13 : SetIt(FId3v2.Images.Pictures.Leaflet,FId3v2.Images.Infos.Leaflet);
        14 : SetIt(FId3v2.Images.Pictures.Lyricist,FId3v2.Images.Infos.Lyricist);
        15 : SetIt(FId3v2.Images.Pictures.Media,FId3v2.Images.Infos.Media);
        16 : SetIt(FId3v2.Images.Pictures.MovieCapture,FId3v2.Images.Infos.MovieCapture);
        17 : SetIt(FId3v2.Images.Pictures.Other,FId3v2.Images.Infos.Other);
        18 : SetIt(FId3v2.Images.Pictures.OtherIcon,FId3v2.Images.Infos.OtherIcon);
        19 : SetIt(FId3v2.Images.Pictures.PublisherLogo,FId3v2.Images.Infos.PublisherLogo);
        20 : SetIt(FId3v2.Images.Pictures.RecordingLocation,FId3v2.Images.Infos.RecordingLocation);
      end;
   end;
end;
{******************************************************************************}
procedure TJvVisualId3v2.SaveImage(Sender: TObject);
begin
   if (FImagePopup.PopupComponent as TImage).Width=0 then
   begin
      beep;
      exit;
   end;
   with TSavePictureDialog.Create(self) do
   begin
      if Execute then
         with (FImagePopup.PopupComponent as TImage) do
            Picture.SaveToFile(FileName);
      free;
   end;
end;
{******************************************************************************}
procedure TJvVisualId3v2.SetFileName(const Value: TFileName);
var
 i:Integer;

  function MediaTypeToStr(Value:TJvID3MediaType):string;
  begin
     case Value of
          mtUNKONW : result:='Unknow';
          mtDIG : result:='Other digital media';
          mtDIG_ANALOG_TRANSFER : result:='Other digital media';
          mtANA : result:='Other analog media';
          mtANA_WAX_CYLINDER : result:='Wax cylinder';
          mtANA_8_TRACK_TAPE : result:='8-track tape cassette';
          mtCD : result:='CD';
          mtCD_ANALOG : result:='CD';
          mtCD_DDD : result:='CD-DDD';
          mtCD_ADD : result:='CD-ADD';
          mtCD_AAD : result:='CD-AAD';
          mtLASERDISC : result:='Laserdisc';
          mtLASERDISC_ANALOG_TRANSFER : result:='Laserdisc';
          mtTURNTABLE : result:='Turntable records';
          mtTURNTABLE_33 : result:='Turntable records - 33.33 rpm';
          mtTURNTABLE_45 : result:='Turntable records - 45 rpm';
          mtTURNTABLE_71 : result:='Turntable records - 71.29 rpm';
          mtTURNTABLE_76 : result:='Turntable records - 76.59 rpm';
          mtTURNTABLE_78 : result:='Turntable records - 78.26 rpm';
          mtTURNTABLE_80 : result:='Turntable records - 80 rpm';
          mtMINIDISC : result:='MiniDisc';
          mtMINIDISC_ANALOG_TRANSFER : result:='MiniDisc';
          mtDAT : result:='DAT';
          mtDAT_ANALOG_TRANSFER : result:='DAT';
          mtDAT_48KHZ_16B : result:='DAT - standard, 48 kHz/16 bits, linear';
          mtDAT_32KHZ_16B : result:='DAT - mode 2, 32 kHz/16 bits, linear';
          mtDAT_32KHZ_12B : result:='DAT - mode 3, 32 kHz/12 bits, nonlinear, low speed';
          mtDAT_44KHZ_16B : result:='DAT - mode 5, 44.1 kHz/16 bits, linear';
          mtDAT_44KHZ_16B_WIDE : result:='DAT - mode 6, 44.1 kHz/16 bits, ''wide track'' play';
          mtDCC : result:='DCC';
          mtDCC_ANALOG_TRANSFER : result:='DCC';
          mtDVD : result:='DVD';
          mtDVD_ANALOG_TRANSFER : result:='DVD';
          mtTV : result:='Television';
          mtTV_PAL : result:='Television - Pal';
          mtTV_NTSC : result:='Television - Netscam';
          mtTV_SECAM : result:='Television - Secam';
          mtVID : result:='Video';
          mtVID_PAL : result:='Video - Pal';
          mtVID_NTSC : result:='Video - Netscam';
          mtVID_SECAM : result:='Video - Secam';
          mtVID_VHS : result:='Video - VHS';
          mtVID_SVHS : result:='Video - S-VHS';
          mtVID_BETA : result:='Video - BetaMax';
          mtRAD : result:='Radio';
          mtRAD_FM : result:='Radio - FM';
          mtRAD_AM : result:='Radio - AM';
          mtRAD_LW : result:='Radio - LW';
          mtRAD_MW : result:='Radio - MW';
          mtTEL : result:='Telephone';
          mtTEL_ISDN : result:='Telephone - ISDN';
          mtMC : result:='MC';
          mtMC_4 : result:='MC - 4.75 cm/s (normal speed for a two sided cassette)';
          mtMC_9 : result:='MC - 9.5 cm/s';
          mtMC_I : result:='MC - Type I cassette (ferric/normal)';
          mtMC_II : result:='MC - Type II cassette (chrome)';
          mtMC_III : result:='MC - Type III cassette (ferric chrome)';
          mtMC_IV : result:='MC - Type IV cassette (metal)';
          mtREE : result:='Reel';
          mtREE_9 : result:='Reel - 9.5 cm/s';
          mtREE_19 : result:='Reel - 19 cm/s';
          mtREE_38 : result:='Reel - 38 cm/s';
          mtREE_76 : result:='Reel - 76 cm/s';
          mtREE_I : result:='Reel - Type I cassette (ferric/normal)';
          mtREE_II : result:='Reel - Type II cassette (chrome)';
          mtREE_III : result:='Reel - Type III cassette (ferric chrome)';
          mtREE_IV : result:='Reel - Type IV cassette (metal)';
          mtDAT_32KHZ_12B_4CH : result:='DAT - mode 4, 32 kHz/12 bits, 4 channels';
     end;
  end;

  function FileTypeToStr(Value:TJvID3FileType):string;
  begin
     case Value of
        ftUNKNOW : result:='Unknow';
        ftMPG : result:='MPEG Audio';
        ftMPG1 : result:='MPEG 1/2 layer I';
        ftMPG2 : result:='MPEG 1/2 layer II';
        ftMPG3 : result:='MPEG 1/2 layer III';
        ftMPG2_5 : result:='MPEG 2.5';
        ftMPG_AAC : result:='Advanced audio compression';
        ftVQF : result:='VQF';
        ftPCM : result:='Pulse Code Modulated audio';
     end;
  end;

  procedure SetBoolean(FLabel:TLabel;Value:boolean);
  begin
     if Value then
       FLabel.Caption:='True'
     else
       FLabel.Caption:='False';
  end;

  procedure AddPhoto(Value:TPicture;Text:string);
  begin
     FPhotosTypes.Add(Text);
     if Value.Width<>0 then
        FPhotoTypes.Items.Add(Text);
  end;

  procedure SetUrl(FLabel:TLabel;Value:string);
  begin
     if Value='' then
     begin
        FLabel.Caption:='';
        FLabel.OnClick:=nil;
        FLabel.Cursor:=crDefault
     end
     else
     begin
        FLabel.Caption:=Value;
        FLabel.OnClick:=UrlClick;
        FLabel.Cursor:=crHandPoint;
     end;
  end;

begin
   //Update
   FId3v2.FileName:=Value;

   //General page
   FFileName.Caption:=ExtractFileName(Value);
   FTagSize.Caption:=IntToStr(FId3v2.TagSize)+' byte(s)';
   FTagVersion.Caption:=Format('%2.2F',[FId3v2.Version]);
   SetBoolean(FUnsynchro,FId3v2.Unsynchronisation);
   SetBoolean(FExperimental,FId3v2.Experimental);
   SetBoolean(FExtended,FId3v2.ExtendedHeader);
   SetBoolean(FPresent,FId3v2.TagPresent);

   //Photos
   FPhotoTypes.Items.Clear;
   FPhotoTypes.OnChange(nil);
   FPhotosTypes.Clear;
   AddPhoto(FId3v2.Images.Pictures.Artist,'Artist');
   AddPhoto(FId3v2.Images.Pictures.Band,'Band');
   AddPhoto(FId3v2.Images.Pictures.BandLogo,'Band Logo');
   AddPhoto(FId3v2.Images.Pictures.ColouredFish,'Coloured Fish');
   AddPhoto(FId3v2.Images.Pictures.Composer,'Composer');
   AddPhoto(FId3v2.Images.Pictures.Conductor,'Conductor');
   AddPhoto(FId3v2.Images.Pictures.CoverBack,'Cover Back');
   AddPhoto(FId3v2.Images.Pictures.CoverFront,'Cover Front');
   AddPhoto(FId3v2.Images.Pictures.DuringPerformance,'During Performance');
   AddPhoto(FId3v2.Images.Pictures.DuringRecording,'During recording');
   AddPhoto(FId3v2.Images.Pictures.FileIcon,'File Icon');
   AddPhoto(FId3v2.Images.Pictures.Illustration,'Illustration');
   AddPhoto(FId3v2.Images.Pictures.LeadArtist,'Lead Artist');
   AddPhoto(FId3v2.Images.Pictures.Leaflet,'Leaflet');
   AddPhoto(FId3v2.Images.Pictures.Lyricist,'Lyricist');
   AddPhoto(FId3v2.Images.Pictures.Media,'Media');
   AddPhoto(FId3v2.Images.Pictures.MovieCapture,'Movie Capture');
   AddPhoto(FId3v2.Images.Pictures.Other,'Other');
   AddPhoto(FId3v2.Images.Pictures.OtherIcon,'Other Icon');
   AddPhoto(FId3v2.Images.Pictures.PublisherLogo,'Publisher Logo');
   AddPhoto(FId3v2.Images.Pictures.RecordingLocation,'Recording Location');
   if FPhototypes.Items.Count>0 then
   begin
      FPhotoTypes.Tag:=-1;
      FPhototypes.ItemIndex:=0;
      FPhotoTypes.OnChange(nil);
   end;

   //Urls
   SetUrl(FCommercial,FId3v2.Web.CommercialInfo);
   SetUrl(FCopyright,FId3v2.Web.LegalInfo);
   SetUrl(FAudioFile,FId3v2.Web.OfficialAudio);
   SetUrl(FArtistPage,FId3v2.Web.OfficialArtist);
   SetUrl(FAudioSource,FId3v2.Web.OfficialAudioSource);
   SetUrl(FRadioStation,FId3v2.Web.InternetRadioStation);
   SetUrl(FPayment,FId3v2.Web.Payment);
   SetUrl(FPublishers,FId3v2.Web.Publishers);
   FUserD.Items.Clear;
   for i:=0 to FId3v2.UserDefinedWeb.ItemCount-1 do
   begin
      FId3v2.UserDefinedWeb.ItemIndex:=i;
      FUserD.Items.Add(FId3v2.UserDefinedWeb.URL);
   end;
   if FUserD.items.Count>0 then
     FUserD.ItemIndex:=0;

   //Involved
   FInvolved.Items.Clear;
   for i:=0 to FId3v2.InvolvedPeople.ItemCount-1 do
   begin
      FId3v2.InvolvedPeople.ItemIndex:=i;
      FInvolved.Items.Add(FId3v2.InvolvedPeople.Person+' ('+FId3v2.InvolvedPeople.Job+')');
   end;
   if FInvolved.Items.Count>0 then
     FInvolved.ItemIndex:=0;
   FSeller.Caption :=FId3v2.Owner.Seller;
   FPrice.Caption  :=FId3v2.Owner.Price;
   FDatePur.Caption:=DateToStr(FId3v2.Owner.DatePurchased);


   //Text part1
   FCreation.Caption:=FId3v2.Texts.Date;
   FTitle.Caption:=FId3v2.Texts.Album;
   FSubTitle.Caption:=FId3v2.Texts.SubTitle;
   FBand.Caption:=FId3v2.Texts.Band;
   FConductor.Caption:=FId3v2.Texts.Conductor;
   FRemixed.Caption:=FId3v2.Texts.ModifiedBy;
   FArtists.Text:=FId3v2.Texts.Performers.Text;
   if FArtists.Items.Count>0 then FArtists.ItemIndex:=0;
   FPublishersLst.Text:=FId3v2.Texts.Composer.Text;
   if FPublishersLst.Items.Count>0 then FPublishersLst.ItemIndex:=0;
   FLanguages.Text:=FId3v2.Texts.Languages.Text;
   if FLanguages.Items.Count>0 then FLanguages.ItemIndex:=0;
   FLyricists.Text:=FId3v2.Texts.Lyricists.Text;
   if FLyricists.Items.Count>0 then FLyricists.ItemIndex:=0;

   //Text part 2
   FContent.Caption:=FId3v2.Texts.Content;
   FAlbum.Caption:=FId3v2.Texts.Album;
   FPart.Caption:=FId3v2.Texts.PartOf;
   FTrack.Caption:=FId3v2.Texts.TrackNumber;
   FISRC.Caption:=FId3v2.Texts.ISRC;
   FYear.Caption:=FId3v2.Texts.Year;
   FDate.Caption:=FId3v2.Texts.Date;
   FTime.Caption:=FId3v2.Texts.Time;
   FRecordedAt.Caption:=FId3v2.Texts.RecordingDate;
   FMediaType.Caption:=MediaTypeToStr(FId3v2.Texts.MediaType);

   //Text part 3
   FBPM.Caption:=IntToStr(FId3v2.Texts.BPM);
   FCopyrightTxt.Caption:=FId3v2.Texts.Copyright;
   FPublisher.Caption:=FId3v2.Texts.Publisher;
   FEncoded.Caption:=FId3v2.Texts.EncodedBy;
   FEncoder.Caption:=FId3v2.Texts.EncodingSoftware;
   FOFileName.Caption:=FId3v2.Texts.OriginalFileName;
   FLength.Caption:=FId3v2.Texts.Length;
   FSize.Caption:=IntToStr(FId3v2.Texts.Size)+' byte(s)';
   FDelay.Caption:=IntToStr(FId3v2.Texts.PlaylistDelay)+' millisecond(s)';
   FInitialKey.Caption:=FId3v2.Texts.InitialKey;

   //Text part 4
   FOAlbum.Caption:=FId3v2.Texts.OriginalTitle;
   FOArtists.Text:=FId3v2.Texts.OriginalArtist.Text;
   if FOArtists.Items.Count>0 then FOArtists.ItemIndex:=0;
   FOLyricists.Text:=FId3v2.Texts.OriginalLyricists.Text;
   if FOLyricists.Items.Count>0 then FOLyricists.ItemIndex:=0;
   FOYear.Caption:=FId3v2.Texts.OriginalReleaseYear;
   FFileOwner.Caption:=FId3v2.Texts.FileOwner;
   FFileType.Caption:=FileTypeToStr(FId3v2.Texts.FileType);
   FRadioName.Caption:=FId3v2.Texts.InternetRadioName;
   FRadioOwner.Caption:=FId3v2.Texts.InternetRadioOwner;

   FUserTexts.Clear;
   for i:=0 to FId3v2.UserDefinedText.ItemCount-1 do
   begin
      FId3v2.UserDefinedText.ItemIndex:=i;
      FUserTexts.Items.Add(FId3v2.UserDefinedText.Value+' ('+FId3v2.UserDefinedText.Description+')');
   end;
   if FUserTexts.Items.Count>0 then FUserTexts.ItemIndex:=0;

   //Popularimeter
   FCounter.Caption:=IntToStr(FId3v2.Popularimeter.Counter);
   FRating.Caption:=IntToStr(FId3v2.Popularimeter.Rating div 255)+'%';
   FUserMail.Caption:=FId3v2.Popularimeter.UserEmail;
end;
{******************************************************************************}
procedure TJvVisualId3v2.StretchImage(Sender: TObject);
begin
   (FImagePopup.PopupComponent as TImage).Stretch:=not((FImagePopup.PopupComponent as TImage).Stretch);
end;
{******************************************************************************}
procedure TJvVisualId3v2.TreeChange(Sender: TObject; Node: TTreeNode);
var
  i:Integer;
begin
   for i:=0 to FPageControl.PageCount-1 do
     if FPageControl.Pages[i].Caption=Node.Text then
       FPageControl.ActivePage:=FPageControl.Pages[i];
end;
{******************************************************************************}
procedure OpenObject(sObjectPath : PChar);
begin
   ShellExecute(0, Nil, sObjectPath, Nil, Nil, SW_NORMAL);
end;
{******************************************************************************}
procedure SurfTo(FUrl:string);
var
   Tempstring : array[0..79] of char;
begin
   StrPCopy(Tempstring,FUrl);
   OpenObject(Tempstring);
end;
{******************************************************************************}
procedure TJvVisualId3v2.UrlClick(Sender: TObject);
begin
   with (Sender as TLabel) do
    SurfTo(Caption);
end;
{******************************************************************************}
procedure TJvVisualId3v2.UrlCombo(Sender: TObject);
begin
   with (Sender as TCombobox) do
    if ItemIndex<>-1 then
      SurfTo(Items[ItemIndex]);
end;
{******************************************************************************}

///////////////////////////////////////////////////////////
// TJvPId3v2Editor
///////////////////////////////////////////////////////////

{******************************************************************************}
constructor TJvPId3v2Editor.Create;
begin
   FInformations:=TJvPInformation.Create;
   FMisc:=TJvPMisc.Create;
   FTexts:=TJvPTexts.Create;
   FUrls:=TJvPUrls.Create;
end;
{******************************************************************************}
destructor TJvPId3v2Editor.Destroy;
begin
   FInformations.Free;
   FMisc.Free;
   FTexts.Free;
   FUrls.Free;
   inherited;
end;
{******************************************************************************}

///////////////////////////////////////////////////////////
// TJvPInformation
///////////////////////////////////////////////////////////

{******************************************************************************}
function TJvPInformation.GetExperimental: string;
begin
   result:=(FParent as TJvVisualId3v2).FExperimental.Caption;
end;
{******************************************************************************}
function TJvPInformation.GetExtended: string;
begin
   result:=(FParent as TJvVisualId3v2).FExtended.Caption;
end;
{******************************************************************************}
function TJvPInformation.GetFileName: string;
begin
   result:=(FParent as TJvVisualId3v2).FFileName.Caption;
end;
{******************************************************************************}
function TJvPInformation.GetPresent: string;
begin
   result:=(FParent as TJvVisualId3v2).FPresent.Caption;
end;
{******************************************************************************}
function TJvPInformation.GetSize: string;
begin
   result:=(FParent as TJvVisualId3v2).FTagSize.Caption;
end;
{******************************************************************************}
function TJvPInformation.GetSynchro: string;
begin
   result:=(FParent as TJvVisualId3v2).FUnsynchro.Caption;
end;
{******************************************************************************}
function TJvPInformation.GetTagVersion: string;
begin
   result:=(FParent as TJvVisualId3v2).FTagVersion.Caption;
end;
{******************************************************************************}
procedure TJvPInformation.SetExperimental(const Value: string);
begin
   (FParent as TJvVisualId3v2).FExperimental.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPInformation.SetExtended(const Value: string);
begin
   (FParent as TJvVisualId3v2).FExtended.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPInformation.SetFileName(const Value: string);
begin
   (FParent as TJvVisualId3v2).FFileName.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPInformation.SetPresent(const Value: string);
begin
   (FParent as TJvVisualId3v2).FPresent.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPInformation.SetSize(const Value: string);
begin
   (FParent as TJvVisualId3v2).FTagSize.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPInformation.SetSynchro(const Value: string);
begin
   (FParent as TJvVisualId3v2).FUnsynchro.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPInformation.SetTagVersion(const Value: string);
begin
   (FParent as TJvVisualId3v2).FTagVersion.Caption:=Value;
end;
{******************************************************************************}

///////////////////////////////////////////////////////////
// TJvPUrls
///////////////////////////////////////////////////////////

{******************************************************************************}
function TJvPUrls.GetArtistPage: string;
begin
   result:=(FParent as TJvVisualId3v2).FArtistPage.Caption;
end;
{******************************************************************************}
function TJvPUrls.GetAudioFile: string;
begin
   result:=(FParent as TJvVisualId3v2).FAudioFile.Caption;
end;
{******************************************************************************}
function TJvPUrls.GetAudioSource: string;
begin
   result:=(FParent as TJvVisualId3v2).FAudioSource.Caption;
end;
{******************************************************************************}
function TJvPUrls.GetCommercial: string;
begin
   result:=(FParent as TJvVisualId3v2).FCommercial.Caption;
end;
{******************************************************************************}
function TJvPUrls.GetCopyright: string;
begin
   result:=(FParent as TJvVisualId3v2).FCommercial.Caption;
end;
{******************************************************************************}
function TJvPUrls.GetPayment: string;
begin
   result:=(FParent as TJvVisualId3v2).FPayment.Caption;
end;
{******************************************************************************}
function TJvPUrls.GetPublishers: string;
begin
   result:=(FParent as TJvVisualId3v2).FPublishers.Caption;
end;
{******************************************************************************}
function TJvPUrls.GetRadioStation: string;
begin
   result:=(FParent as TJvVisualId3v2).FRadioStation.Caption;
end;
{******************************************************************************}
function TJvPUrls.GetUserDefined: TstringList;
begin
   result:=TstringList.Create;
   result.Text:=(FParent as TJvVisualId3v2).FUserD.Items.Text;
end;
{******************************************************************************}
procedure TJvPUrls.SetArtistPage(const Value: string);
begin
   (FParent as TJvVisualId3v2).FArtistPage.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPUrls.SetAudioFile(const Value: string);
begin
   (FParent as TJvVisualId3v2).FAudioFile.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPUrls.SetAudioSource(const Value: string);
begin
   (FParent as TJvVisualId3v2).FAudioSource.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPUrls.SetCommercial(const Value: string);
begin
   (FParent as TJvVisualId3v2).FCommercial.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPUrls.SetCopyright(const Value: string);
begin
   (FParent as TJvVisualId3v2).FCopyright.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPUrls.SetPayment(const Value: string);
begin
   (FParent as TJvVisualId3v2).FPayment.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPUrls.SetPublishers(const Value: string);
begin
   (FParent as TJvVisualId3v2).FPublishers.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPUrls.SetRadioStation(const Value: string);
begin
   (FParent as TJvVisualId3v2).FRadioStation.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPUrls.SetUserDefined(const Value: TstringList);
begin
   (FParent as TJvVisualId3v2).FUserD.Items.Text:=Value.Text;
end;
{******************************************************************************}

///////////////////////////////////////////////////////////
// TJvPMisc
///////////////////////////////////////////////////////////

{******************************************************************************}
function TJvPMisc.GetCounter: string;
begin
   result:=(FParent as TJvVisualId3v2).FCounter.Caption;
end;
{******************************************************************************}
function TJvPMisc.GetInvolved: TstringList;
begin
   result:=TstringList.Create;
   result.Text:=(FParent as TJvVisualId3v2).FInvolved.Text;
end;
{******************************************************************************}
function TJvPMisc.GetPrice: string;
begin
   result:=(FParent as TJvVisualId3v2).FPrice.Caption;
end;
{******************************************************************************}
function TJvPMisc.GetPurchased: string;
begin
   result:=(FParent as TJvVisualId3v2).FDatePur.Caption;
end;
{******************************************************************************}
function TJvPMisc.GetRating: string;
begin
   result:=(FParent as TJvVisualId3v2).FRating.Caption;
end;
{******************************************************************************}
function TJvPMisc.GetSeller: string;
begin
   result:=(FParent as TJvVisualId3v2).FSeller.Caption;
end;
{******************************************************************************}
function TJvPMisc.GetUserEmail: string;
begin
   result:=(FParent as TJvVisualId3v2).FUserMail.Caption;
end;
{******************************************************************************}
procedure TJvPMisc.SetCounter(const Value: string);
begin
   (FParent as TJvVisualId3v2).FCounter.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPMisc.SetInvolved(const Value: TstringList);
begin
   (FParent as TJvVisualId3v2).FInvolved.Text:=Value.Text;
end;
{******************************************************************************}
procedure TJvPMisc.SetPrice(const Value: string);
begin
   (FParent as TJvVisualId3v2).FPrice.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPMisc.SetPurchased(const Value: string);
begin
   (FParent as TJvVisualId3v2).FDatePur.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPMisc.SetRating(const Value: string);
begin
   (FParent as TJvVisualId3v2).FRating.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPMisc.SetSeller(const Value: string);
begin
   (FParent as TJvVisualId3v2).FSeller.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPMisc.SetUserEmail(const Value: string);
begin
   (FParent as TJvVisualId3v2).FUserMail.Caption:=Value;
end;
{******************************************************************************}

///////////////////////////////////////////////////////////
// TJvPTexts
///////////////////////////////////////////////////////////

{******************************************************************************}
constructor TJvPTexts.Create;
begin
   FJvText1:=TJvPText1.Create;
   FJvText2:=TJvPText2.Create;
   FJvText3:=TJvPText3.Create;
   FJvText4:=TJvPText4.Create;
end;
{******************************************************************************}
destructor TJvPTexts.Destroy;
begin
   FJvText1.Free;
   FJvText2.Free;
   FJvText3.Free;
   FJvText4.Free;
   inherited;
end;
{******************************************************************************}

///////////////////////////////////////////////////////////
// TJvPText1
///////////////////////////////////////////////////////////

{******************************************************************************}
function TJvPText1.GetArtists: TstringList;
begin
   result:=TstringList.Create;
   result.Text:=(FParent as TJvVisualId3v2).FArtists.Text;
end;
{******************************************************************************}
function TJvPText1.GetBand: string;
begin
   result:=(FParent as TJvVisualId3v2).FBand.Caption;
end;
{******************************************************************************}
function TJvPText1.GetComposers: TstringList;
begin
   result:=TstringList.Create;
   result.Text:=(FParent as TJvVisualId3v2).FPublishersLst.Text;
end;
{******************************************************************************}
function TJvPText1.GetConductor: string;
begin
   result:=(FParent as TJvVisualId3v2).FConductor.Caption;
end;
{******************************************************************************}
function TJvPText1.GetCreation: string;
begin
   result:=(FParent as TJvVisualId3v2).FCreation.Caption;
end;
{******************************************************************************}
function TJvPText1.GetLanguages: TstringList;
begin
   result:=TstringList.Create;
   result.Text:=(FParent as TJvVisualId3v2).FLanguages.Text;
end;
{******************************************************************************}
function TJvPText1.GetLyricists: TstringList;
begin
   result:=TstringList.Create;
   result.Text:=(FParent as TJvVisualId3v2).FLyricists.Text;
end;
{******************************************************************************}
function TJvPText1.GetRemixed: string;
begin
   result:=(FParent as TJvVisualId3v2).FRemixed.Caption;
end;
{******************************************************************************}
function TJvPText1.GetSubTitle: string;
begin
   result:=(FParent as TJvVisualId3v2).FSubTitle.Caption;
end;
{******************************************************************************}
function TJvPText1.GetTitle: string;
begin
   result:=(FParent as TJvVisualId3v2).FTitle.Caption;
end;
{******************************************************************************}
procedure TJvPText1.SetArtists(const Value: TstringList);
begin
   (FParent as TJvVisualId3v2).FArtists.Text:=Value.Text;
end;
{******************************************************************************}
procedure TJvPText1.SetBand(const Value: string);
begin
   (FParent as TJvVisualId3v2).FBand.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText1.SetComposers(const Value: TstringList);
begin
   (FParent as TJvVisualId3v2).FPublishersLst.Text:=Value.Text;
end;
{******************************************************************************}
procedure TJvPText1.SetConductor(const Value: string);
begin
   (FParent as TJvVisualId3v2).FConductor.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText1.SetCreation(const Value: string);
begin
   (FParent as TJvVisualId3v2).FCreation.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText1.SetLanguages(const Value: TstringList);
begin
   (FParent as TJvVisualId3v2).FLanguages.Text:=Value.Text;
end;
{******************************************************************************}
procedure TJvPText1.SetLyricists(const Value: TstringList);
begin
   (FParent as TJvVisualId3v2).FLyricists.Text:=Value.Text;
end;
{******************************************************************************}
procedure TJvPText1.SetRemixed(const Value: string);
begin
   (FParent as TJvVisualId3v2).FRemixed.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText1.SetSubTitle(const Value: string);
begin
   (FParent as TJvVisualId3v2).FSubTitle.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText1.SetTitle(const Value: string);
begin
   (FParent as TJvVisualId3v2).FTitle.Caption:=Value;
end;
{******************************************************************************}

///////////////////////////////////////////////////////////
// TJvPText2
///////////////////////////////////////////////////////////

{******************************************************************************}
function TJvPText2.GetAlbum: string;
begin
   result:=(FParent as TJvVisualId3v2).FAlbum.Caption;
end;
{******************************************************************************}
function TJvPText2.GetContentType: string;
begin
   result:=(FParent as TJvVisualId3v2).FContent.Caption;
end;
{******************************************************************************}
function TJvPText2.GeTDate: string;
begin
   result:=(FParent as TJvVisualId3v2).FDate.Caption;
end;
{******************************************************************************}
function TJvPText2.GetISRC: string;
begin
   result:=(FParent as TJvVisualId3v2).FISRC.Caption;
end;
{******************************************************************************}
function TJvPText2.GetMediaType: string;
begin
   result:=(FParent as TJvVisualId3v2).FMediaType.Caption;
end;
{******************************************************************************}
function TJvPText2.GetPart: string;
begin
   result:=(FParent as TJvVisualId3v2).FPart.Caption;
end;
{******************************************************************************}
function TJvPText2.GetRecorded: string;
begin
   result:=(FParent as TJvVisualId3v2).FRecordedAt.Caption;
end;
{******************************************************************************}
function TJvPText2.GeTTime: string;
begin
   result:=(FParent as TJvVisualId3v2).FTime.Caption;
end;
{******************************************************************************}
function TJvPText2.GetTrack: string;
begin
   result:=(FParent as TJvVisualId3v2).FTrack.Caption;
end;
{******************************************************************************}
function TJvPText2.GetYear: string;
begin
   result:=(FParent as TJvVisualId3v2).FYear.Caption;
end;
{******************************************************************************}
procedure TJvPText2.SetAlbum(const Value: string);
begin
   (FParent as TJvVisualId3v2).FAlbum.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText2.SetContentType(const Value: string);
begin
   (FParent as TJvVisualId3v2).FContent.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText2.SeTDate(const Value: string);
begin
   (FParent as TJvVisualId3v2).FDate.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText2.SetISRC(const Value: string);
begin
   (FParent as TJvVisualId3v2).FISRC.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText2.SetMediaType(const Value: string);
begin
   (FParent as TJvVisualId3v2).FMediaType.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText2.SetPart(const Value: string);
begin
   (FParent as TJvVisualId3v2).FPart.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText2.SetRecorded(const Value: string);
begin
   (FParent as TJvVisualId3v2).FRecordedAt.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText2.SeTTime(const Value: string);
begin
   (FParent as TJvVisualId3v2).FTime.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText2.SetTrack(const Value: string);
begin
   (FParent as TJvVisualId3v2).FTrack.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText2.SetYear(const Value: string);
begin
   (FParent as TJvVisualId3v2).FYear.Caption:=Value;
end;
{******************************************************************************}

///////////////////////////////////////////////////////////
// TJvPText3
///////////////////////////////////////////////////////////

{******************************************************************************}
function TJvPText3.GetBPM: string;
begin
   result:=(FParent as TJvVisualId3v2).FBPM.Caption;
end;
{******************************************************************************}
function TJvPText3.GetCopyright: string;
begin
   result:=(FParent as TJvVisualId3v2).FCopyright.Caption;
end;
{******************************************************************************}
function TJvPText3.GetDelay: string;
begin
   result:=(FParent as TJvVisualId3v2).FDelay.Caption;
end;
{******************************************************************************}
function TJvPText3.GetEncodedBy: string;
begin
   result:=(FParent as TJvVisualId3v2).FEncoded.Caption;
end;
{******************************************************************************}
function TJvPText3.GetEncoder: string;
begin
   result:=(FParent as TJvVisualId3v2).FEncoder.Caption;
end;
{******************************************************************************}
function TJvPText3.GetInitialKey: string;
begin
   result:=(FParent as TJvVisualId3v2).FInitialKey.Caption;
end;
{******************************************************************************}
function TJvPText3.GetLength: string;
begin
   result:=(FParent as TJvVisualId3v2).FLength.Caption;
end;
{******************************************************************************}
function TJvPText3.GetOFileName: string;
begin
   result:=(FParent as TJvVisualId3v2).FOFileName.Caption;
end;
{******************************************************************************}
function TJvPText3.GetPublisher: string;
begin
   result:=(FParent as TJvVisualId3v2).FPublisher.Caption;
end;
{******************************************************************************}
function TJvPText3.GetSize: string;
begin
   result:=(FParent as TJvVisualId3v2).FSize.Caption;
end;
{******************************************************************************}
procedure TJvPText3.SetBPM(const Value: string);
begin
   (FParent as TJvVisualId3v2).FBPM.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText3.SetCopyright(const Value: string);
begin
   (FParent as TJvVisualId3v2).FCopyright.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText3.SetDelay(const Value: string);
begin
   (FParent as TJvVisualId3v2).FDelay.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText3.SetEncodedBy(const Value: string);
begin
   (FParent as TJvVisualId3v2).FEncoded.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText3.SetEncoder(const Value: string);
begin
   (FParent as TJvVisualId3v2).FEncoder.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText3.SetInitialKey(const Value: string);
begin
   (FParent as TJvVisualId3v2).FInitialKey.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText3.SetLength(const Value: string);
begin
   (FParent as TJvVisualId3v2).FLength.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText3.SetOFileName(const Value: string);
begin
   (FParent as TJvVisualId3v2).FOFileName.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText3.SetPublisher(const Value: string);
begin
   (FParent as TJvVisualId3v2).FPublisher.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText3.SetSize(const Value: string);
begin
   (FParent as TJvVisualId3v2).FSize.Caption:=Value;
end;
{******************************************************************************}

///////////////////////////////////////////////////////////
// TJvPText4
///////////////////////////////////////////////////////////

{******************************************************************************}
function TJvPText4.GetFileOwner: string;
begin
   result:=(FParent as TJvVisualId3v2).FFileOwner.Caption;
end;
{******************************************************************************}
function TJvPText4.GetFileType: string;
begin
   result:=(FParent as TJvVisualId3v2).FFileType.Caption;
end;
{******************************************************************************}
function TJvPText4.GetOAlbum: string;
begin
   result:=(FParent as TJvVisualId3v2).FOAlbum.Caption;
end;
{******************************************************************************}
function TJvPText4.GetOArtists: TstringList;
begin
   result:=TstringList.Create;
   result.Text:=(FParent as TJvVisualId3v2).FOArtists.Text;
end;
{******************************************************************************}
function TJvPText4.GetOLyricists: TstringList;
begin
   result:=TstringList.Create;
   result.Text:=(FParent as TJvVisualId3v2).FOLyricists.Text;
end;
{******************************************************************************}
function TJvPText4.GetRadioName: string;
begin
   result:=(FParent as TJvVisualId3v2).FRadioName.Caption;
end;
{******************************************************************************}
function TJvPText4.GetRadioOwner: string;
begin
   result:=(FParent as TJvVisualId3v2).FRadioOwner.Caption;
end;
{******************************************************************************}
function TJvPText4.GetUserDefined: TstringList;
begin
   result:=TstringList.Create;
   result.Text:=(FParent as TJvVisualId3v2).FUserTexts.Text;
end;
{******************************************************************************}
function TJvPText4.GetYear: string;
begin
   result:=(FParent as TJvVisualId3v2).FOYear.Caption;
end;
{******************************************************************************}
procedure TJvPText4.SetFileOwner(const Value: string);
begin
   (FParent as TJvVisualId3v2).FFileOwner.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText4.SetFileType(const Value: string);
begin
   (FParent as TJvVisualId3v2).FFileType.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText4.SetOAlbum(const Value: string);
begin
   (FParent as TJvVisualId3v2).FOAlbum.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText4.SetOArtists(const Value: TstringList);
begin
   (FParent as TJvVisualId3v2).FOArtists.Text:=Value.Text;
end;
{******************************************************************************}
procedure TJvPText4.SetOLyricists(const Value: TstringList);
begin
   (FParent as TJvVisualId3v2).FOLyricists.Text:=Value.Text;
end;
{******************************************************************************}
procedure TJvPText4.SetRadioName(const Value: string);
begin
   (FParent as TJvVisualId3v2).FRadioName.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText4.SetRadioOwner(const Value: string);
begin
   (FParent as TJvVisualId3v2).FRadioOwner.Caption:=Value;
end;
{******************************************************************************}
procedure TJvPText4.SetUserDefined(const Value: TstringList);
begin
   (FParent as TJvVisualId3v2).FUserTexts.Text:=Value.Text;
end;
{******************************************************************************}
procedure TJvPText4.SetYear(const Value: string);
begin
   (FParent as TJvVisualId3v2).FOYear.Caption:=Value;
end;
{******************************************************************************}

///////////////////////////////////////////////////////////
// TJvPId3v2TreeView
///////////////////////////////////////////////////////////

{******************************************************************************}
function TJvPId3v2TreeView.GetHideSelection: boolean;
begin
   result:=(FParent as TJvVisualId3v2).FTreeView.HideSelection;
end;
{******************************************************************************}
function TJvPId3v2TreeView.GetHotTrack: boolean;
begin
   result:=(FParent as TJvVisualId3v2).FTreeView.HotTrack;
end;
{******************************************************************************}
function TJvPId3v2TreeView.GetReadOnly: boolean;
begin
   result:=(FParent as TJvVisualId3v2).FTreeView.ReadOnly;
end;
{******************************************************************************}
function TJvPId3v2TreeView.GetWidth: Integer;
begin
   result:=(FParent as TJvVisualId3v2).FTreeView.Width;
end;
{******************************************************************************}
procedure TJvPId3v2TreeView.SetHideSelection(const Value: boolean);
begin
   (FParent as TJvVisualId3v2).FTreeView.HideSelection:=Value;
end;
{******************************************************************************}
procedure TJvPId3v2TreeView.SetHotTrack(const Value: boolean);
begin
   (FParent as TJvVisualId3v2).FTreeView.HotTrack:=Value;
end;
{******************************************************************************}
procedure TJvPId3v2TreeView.SetReadOnly(const Value: boolean);
begin
   (FParent as TJvVisualId3v2).FTreeView.ReadOnly:=Value;
end;
{******************************************************************************}
procedure TJvPId3v2TreeView.SetWidth(const Value: integer);
begin
   (FParent as TJvVisualId3v2).FTreeView.Width:=Value;
end;
{******************************************************************************}

end.
