{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvWinHelp.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvWinHelp;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, JvTypes;

type
  TJvWinHelp = class(TComponent)
  private
    FHelpFile: string;
    FOwner:TComponent;
    function GetHelpFile:PChar;
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property HelpFile:string read FHelpFile write FHelpFile;
    function ShowContextualHelp(Control:TWinControl):boolean;
    function ExecuteCommand(MacroCommand:string):boolean;
    function ShowHelp(Control:TWinControl):boolean;
    function ShowContents:boolean;
    function ShowHelpOnHelp:boolean;
    function ShowIndex:boolean;
    function ShowKeyword(Keyword:string):boolean;
    function ShowPartialKeyWord(Keyword:string):boolean;
    function SetWindowPos(Left,Top,Width,Height:Integer;Visibility:integer):boolean;
  end;

implementation

resourcestring
  RC_OwnerForm     =     'Owner must be of type TForm';

{**************************************************}
constructor TJvWinHelp.Create(AOwner: TComponent);
begin
   inherited;
   FHelpFile:='';
   FOwner:=AOwner;

   while FOwner.GetParenTComponent<>nil do
      FOwner:=FOwner.GetParenTComponent;
   if not(FOwner is TForm) then
      Raise Exception.Create(RC_OwnerForm);
end;
{**************************************************}
destructor TJvWinHelp.Destroy;
begin
   WinHelp(TWinControl(FOwner).Handle,GetHelpFile,HELP_QUIT,0);
   inherited;
end;
{**************************************************}
function TJvWinHelp.ExecuteCommand(MacroCommand: string): boolean;
begin
   result:=WinHelp(TWinControl(FOwner).Handle,GetHelpFile,HELP_COMMAND,Longint(PChar(MacroCommand)));
end;
{**************************************************}
function TJvWinHelp.GetHelpFile: PChar;
begin
   if FHelpFile='' then
     result:=PChar((FOwner as TForm).HelpFile)
   else
     result:=PChar(FHelpFile);
end;
{**************************************************}
function TJvWinHelp.SetWindowPos(Left, Top, Width, Height,
  Visibility: integer): boolean;
var
  HelpInfo:HELPWININFO;
begin
   HelpInfo.wStructSize:=SizeOf(HELPWININFO);
   HelpInfo.x:=Left;
   HelpInfo.y:=Top;
   HelpInfo.dx:=Width;
   HelpInfo.dy:=Height;
   HelpInfo.wMax:=Visibility;

   result:=WinHelp(TWinControl(FOwner).Handle,GetHelpFile,HELP_SETWINPOS,Longint(@HelpInfo));
end;
{**************************************************}
function TJvWinHelp.ShowContents: boolean;
begin
   result:=WinHelp(TWinControl(FOwner).Handle,GetHelpFile,HELP_CONTENTS,0);
end;
{**************************************************}
function TJvWinHelp.ShowContextualHelp(Control: TWinControl):boolean;
begin
   result:=WinHelp(TWinControl(FOwner).Handle,GetHelpFile,HELP_CONTEXTPOPUP,Control.HelpContext);
end;
{**************************************************}
function TJvWinHelp.ShowHelp(Control: TWinControl): boolean;
begin
   result:=WinHelp(TWinControl(FOwner).Handle,GetHelpFile,HELP_CONTEXT,Control.HelpContext);
end;
{**************************************************}
function TJvWinHelp.ShowHelpOnHelp: boolean;
begin
   result:=WinHelp(TWinControl(FOwner).Handle,GetHelpFile,HELP_HELPONHELP,0);
end;
{**************************************************}
function TJvWinHelp.ShowIndex: boolean;
begin
   result:=WinHelp(TWinControl(FOwner).Handle,GetHelpFile,HELP_INDEX,0);
end;
{**************************************************}
function TJvWinHelp.ShowKeyword(Keyword: string): boolean;
begin
   result:=WinHelp(TWinControl(FOwner).Handle,GetHelpFile,HELP_KEY,Longint(PChar(KeyWord)));
end;
{**************************************************}
function TJvWinHelp.ShowPartialKeyWord(Keyword: string): boolean;
begin
   result:=WinHelp(TWinControl(FOwner).Handle,GetHelpFile,HELP_PARTIALKEY,Longint(PChar(KeyWord)));
end;
{**************************************************}
end.
