{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvWinampLabel.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvWinampLabel;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, StdCtrls;

const
  ScrollBy = 1;
  {$EXTERNALSYM ScrollBy}
  Row1 : string[31] = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ"@';
  {$EXTERNALSYM Row1}
  Row2 : string[31] = '0123456789. :()-''!_+\/[]^&%.=$#';
  {$EXTERNALSYM Row2}
  Row3 : string[31] = '?* ';
  {$EXTERNALSYM Row3}

type
  TJvWinampThread = class(TThread)
  protected
    procedure Draw;
    procedure Execute; override;
  public
    FDelay:cardinal;
    FOnDraw:TNotifyEvent;
  end;

  TJvWinampLabel = class(TCustomLabel)
  private
    FBitmap     : TBitmap;
    FPicture    : TPicture;
    FTimer      : TJvWinampThread;
    FInterval   : Cardinal;
    FActive     : Boolean;
    FStretch    : Boolean;
    FScrollBy   : Integer;
    FCurPos     : Integer;
    FWait       : Integer;
    FWaiting    : Boolean;
    FColor      : TColor;
    FCaption    : string;
    FScale      : Real;
    function GetCol(Ch: Char):Word;
    procedure SeTColor(Value: TColor);
    function  GetScrollBy:Integer;
    procedure SetActive(Value: Boolean);
    procedure SetStretch(Value: Boolean);
    procedure SetInterval(Value: Cardinal);
    procedure SetPicture(Value: TPicture);
    procedure FillBitMap;
    procedure Activate;
    procedure Deactivate;
    procedure UpdatePos;
    procedure DoOnTimer(Sender: TObject);
    function GetRow(Ch: Char):Word;
    procedure SetText(Value: string);
  protected
    procedure paint; override;
  public
    CharWidth,CharHeight: Integer;
    constructor Create(AOwner: TComponent);override;
    destructor Destroy; override;
  published
    property Active: Boolean read FActive write SetActive;
    property Stretch: Boolean read FStretch write SetStretch;
    property Scroll_By: Integer read GetScrollBy write FScrollBy;
    property Scroll_Interval : Cardinal read FInterval write SetInterval;
    property WaitOnend: Integer read FWait write FWait;
    property Skin: TPicture read FPicture write SetPicture;
    property Color: TColor read FColor write SeTColor;
    property Text: string read FCaption write SetText;
    property Align;
    property Alignment;
    property FocusControl;
    property DragCursor;
    property DragMode;
    property ParentColor;
    property ShowHint;
    property ParentShowHint;
    property Layout;
    property Left;
    property Transparent;
    property PopupMenu;
    property Visible;
    property Top;
    property Height;
    property Width;
    property Cursor;
    property Enabled;
    property Hint;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseMove;
    property OnMouseDown;
    property OnMouseUp;
    property OnStartDrag;
  end;

implementation

resourcestring
   RC_InvalidSkin               =       'Invalid skin';
   RC_WinampRC                  =       'WINAMP1';

{$R RES_WinampLabel.res}

{**************************************************}
procedure TJvWinampThread.Draw;
begin
  if Assigned(FOnDraw) then
     FOnDraw(self);
end;
{**************************************************}
procedure TJvWinampThread.Execute;
begin
  while not(Terminated) do
  begin
    Sleep(FDelay);
    Synchronize(Draw);
  end;
end;
{**************************************************}



{**************************************************}
constructor TJvWinampLabel.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  AutoSize:=False;
  FInterval:=100;
  charwidth:=5;
  charheight:=6;
  FPicture:=TPicture.Create;
  FPicture.Bitmap.LoadFromResourceName(HInstance,RC_WinampRC);
  FBitmap:=TBitmap.Create;
  with FBitmap do
  begin
    pixelformat:=pf24bit;
    width:=10;
    height:=10;
  end;
  FTimer:=TJvWinampThread.Create(true);
  with FTimer do
  begin
     FreeOnTerminate:=true;
     FDelay:=FInterval;
     FOnDraw:=DoOnTimer;
  end;
  Width:=100;
  Height:=CharHeight*2;
  FActive:=False;
  Activate;
  FStretch:=True;
  FScrollBy:=2;
  FWait:=1000;
  Color:=clBlack;
end;
{**************************************************}
destructor TJvWinampLabel.Destroy;
begin
  Deactivate;
  FBitmap.Free;
  FPicture.Free;
  FTimer.Free;
  inherited Destroy;
end;
{**************************************************}
function TJvWinampLabel.GetScrollBy:Integer;
begin
  Result:=ABS(FScrollBy);
end;
{**************************************************}
procedure TJvWinampLabel.SetPicture(Value: TPicture);
begin
  FPicture.Assign(Value);
  if (FPicture.Bitmap.Width<>155) or (FPicture.Bitmap.Height<>18) then
    raise Exception.Create(RC_InvalidSkin);
  FCaption:='';
  Invalidate;
end;
{**************************************************}
procedure TJvWinampLabel.SeTColor(Value : TColor);
begin
  if Value<>FColor then
  begin
    FColor:=Value;
    FCaption:='';
    Invalidate;
  end;
end;
{**************************************************}
procedure TJvWinampLabel.SetActive(Value : Boolean);
begin
  if Value <> FActive then
  begin
    FActive:=Value;
    if FActive then
      Activate
    else
      Deactivate;
    FWaiting:=False;
  end;
end;
{**************************************************}
procedure TJvWinampLabel.SetStretch(Value : Boolean);
var
 Rec : TRect;
begin
  if Value<>FStretch then
  begin
    FStretch:=Value;
    Rec.Top:=0; Rec.Left:=0; Rec.Bottom:=Height; Rec.Right:=Width;
    Canvas.Brush.Color:=Color;
    Canvas.Brush.style:=bsSolid;
    Canvas.FillRect(Rec);
    Paint;
  end;
  if not FStretch then
    FScale:=1;
end;
{**************************************************}
procedure TJvWinampLabel.SetInterval(Value : Cardinal);
begin
  if Value <> FInterval then
  begin
    FInterval:=Value;
    FTimer.FDelay:=Value;
  end;
end;
{**************************************************}
procedure TJvWinampLabel.Activate;
begin
  FActive:=True;
  if not(csDesigning in ComponentState) then
    FTimer.Resume;
  FTimer.FDelay:=FInterval;
  FWaiting:=False;

  FCurPos:=0;
  FScrollBy:=ABS(FScrollBy);
  FillBitMap;
end;
{**************************************************}
procedure TJvWinampLabel.Deactivate;
begin
  FTimer.Suspend;
  FActive:=False;
  Invalidate;
end;
{**************************************************}
procedure TJvWinampLabel.DoOnTimer(Sender : TObject);
begin
  if FWaiting then
  begin
     FTimer.FDelay:=FInterval;
     FWaiting:=False;
  end;
  UpDatePos;
  Paint;
end;
{**************************************************}
function TJvWinampLabel.GetCol(Ch: Char):Word;
var
 index: Integer;
begin
  Ch := UpCase(Ch);
  index := pos(Ch, Row1);
  if index=-1 then
    index := pos(Ch,row2);
  if index=-1 then
    index := pos(Ch,row3);
  if index=-1 then
    result := GetCol(' ')
  else
    result := index-1*CharWidth;
end;
{**************************************************}
function TJvWinampLabel.GetRow(Ch: Char):Word;
begin
  Ch := UpCase(Ch);
  result :=0;
  if pos(Ch,Row2)<>0 then
    result := CharHeight
  else if pos(Ch,Row3)<>0 then
    result := 2*CharHeight;
end;
{**************************************************}
procedure TJvWinampLabel.FillBitmap;
Var
 Rec: TRect;
 T: Word;
begin
  try
   with FBitmap do
   begin
     TransparentMode:=tmAuto;
     if Caption<>'' then
       Width := (Length(Caption)*CharWidth)
     else
       Width := self.Width;
     Height := CharHeight;
     if Width < self.Width then
       Width := self.Width;
     Rec.Top:=0;
     Rec.Left:=0;
     Rec.Bottom:=Height;
     Rec.Right:=Width;
     Canvas.Brush.Color := Color;
     Canvas.Brush.Style := bsSolid;
     Canvas.FillRect(Rec);
     if self.Caption<>'' then
       for T:=0 to Length(Caption)-1 do
         BitBlt(Canvas.Handle,T*CharWidth,0,CharWidth,CharHeight,
           Canvas.Handle,GetCol(Caption[T+1]),GetRow(Caption[T+1]),SrcCopy);
   end;
  except
  end;
end;
{**************************************************}
procedure TJvWinampLabel.UpDatePos;
begin
   try
     if (Length(Self.Caption)*CharWidth)*FScale > Self.Width then
     begin
        FCurPos:=FCurPos+FScrollBy;
        if FCurPos <= 0 then
        begin
           FScrollBy:=Abs(FScrollBy);
           if FWait<>0 then
           begin
              FWaiting:=True;
              FTimer.FDelay:=FWait;
           end;
        end;
        if (Length(Self.Caption)*CharWidth-(FCurPos)) <= (Self.Width/FScale) then
        begin
           FScrollBy:=Abs(FScrollBy)*-1;
           if FWait<>0 then
           begin
              FWaiting:=True;
              FTimer.FDelay:=FWait;
           end;
        end;
     end
     else FCurPos:=0;
   except
   end;
end;
{**************************************************}
procedure TJvWinampLabel.Paint;
Var
 Rec : TRect;
begin
   try
      if not FStretch then
      begin
         Rec:=ClientRect;
         Rec.Top:=Rec.Top+CharHeight;
         Canvas.FillRect(Rec);
         if FActive then
           BitBlt(Canvas.Handle,0,0,Width,CharHeight,FBitmap.Canvas.Handle,FCurPos,0,srcCopy)
         else
         begin
            Rec:=ClientRect;
            Rec.Bottom:=Rec.Bottom+CharHeight;
            Rec.Left:=rec.Left+(CharWidth*Length(Caption));
            Canvas.FillRect(Rec);
            BitBlt(Canvas.Handle,0,0,Width,CharHeight,FBitmap.Canvas.Handle,0,0,srcCopy);
         end;
      end
      else
      begin
         FScale:=Height/CharHeight;
         if FActive then
            StretchBlt(Canvas.Handle,0,0,Width,Height,FBitmap.Canvas.Handle,FCurPos,0,Round(Width/FScale),CharHeight,srcCopy)
         else
            StretchBlt(Canvas.Handle,0,0,Width,Height,FBitmap.Canvas.Handle,0,0,Round(Width/FScale),CharHeight,srcCopy);
      end;
   except
   end;
end;
{**************************************************}
procedure TJvWinampLabel.SetText(Value:string);
Var
 Rec : TRect;
begin
   if Value<>FCaption then
   begin
      Caption:=Value;
      FillBitmap;
      Rec.Top:=0; Rec.Left:=0; Rec.Bottom:=Height; Rec.Right:=Width;
      Canvas.Brush.Color:=Color;
      Canvas.Brush.style:=bsSolid;
      Canvas.FillRect(Rec);
      FCurPos:=0;
      FScrollBy:=ABS(FScrollBy);
      FCaption:=Caption;
   end;
end;
{**************************************************}
end.
