{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: JvZoom.PAS, released on 2001-02-28.

The Initial Developer of the Original Code is Sbastien Buysse [sbuysse@buypin.com]
Portions created by Sbastien Buysse are Copyright (C) 2001 Sbastien Buysse.
All Rights Reserved.

Contributor(s): Michael Beck [mbeck@bigfoot.com].

Last Modified: 2000-02-28

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org

Known Issues:
-----------------------------------------------------------------------------}
{$A+,B-,C+,D+,E-,F-,G+,H+,I+,J+,K-,L+,M-,N+,O+,P+,Q-,R-,S-,T-,U-,V+,W-,X+,Y+,Z1}
{$I JEDI.INC}

unit JvZoom;

{$ObjExportAll On}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, extctrls;

type
  TJvZoom = class(TWinControl)
  private
    FTimer:TTimer;
    FActive: boolean;
    FZoom: Integer;
    FDelay: cardinal;
    FZCanvas:TControlCanvas;
    FCanvas:TCanvas;
    Flastpoint:tpoint;
    procedure PaintMe(Sender: TObject);
    procedure SetActive(const Value: boolean);
    procedure SetDelay(const Value: cardinal);
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
  protected
  public
    constructor Create(AOwner: TComponent);override;
    destructor Destroy;override;
  published
    property Active:boolean read FActive write SetActive default true;
    property ZoomLevel:Integer read FZoom write FZoom default 100;
    property Delay:cardinal read FDelay write SetDelay default 100;
    property OnMouseDown;
    property OnClick;
    property OnDblClick;
    property OnMouseUp;
    property OnResize;
    property OnKeyPress;
    property OnKeyDown;
    property OnKeyUp;
    property OnMouseWheel;
    property OnMouseWheelUp;
    property OnMouseWheelDown;
  end;

implementation

{**************************************************}
constructor TJvZoom.Create(AOwner: TComponent);
begin
   inherited;
   self.height:=100;
   self.width:=100;
   FDelay:=100;
   FZoom:=100;
   FActive:=true;

   //Create canvas to write to the control
   FZCanvas:=TControlCanvas.Create;
   FZCanvas.Control:=self;

   //Create the canvas to retrieve informations
   FCanvas:=TCanvas.Create;
   FCanvas.Handle:=GetDC(0);

   Ftimer:=TTimer.Create(self);
   FTimer.OnTimer:=PaintMe;
   FTimer.Interval:=100;

   //Assign the parent, or it's impossible to draw on it
   self.parent:=TWinControl(AOwner);

   FTimer.enabled:=true;
end;
{**************************************************}
destructor TJvZoom.Destroy;
begin
   ReleaseDc(0,FCanvas.handle);
   FCanvas.free;
   FZCanvas.free;
   inherited;
end;
{**************************************************}
procedure TJvZoom.PaintMe(Sender: TObject);
var
  p:Tpoint;
  x,y:Integer;
  t:Trect;
begin
   GetCursorPos(p);
   //Only draw if on a different position
   if (p.x<>FLastPoint.x)or(p.y<>FLastPoint.y) then
   begin
      //if it's the first time, get the DC of the control to be able
      //to draw on it.
      if FTimer.tag=0 then
      begin
         try
            FZCanvas.Handle:=GetDc(self.handle);
         except
            exit;
         end;
         FTimer.tag:=1;
      end;

      //Analyse the point
      FLastPoint:=p;
      //Create the area to Copy
      x:=(self.width div 2)*Fzoom div 100;
      y:=(self.height div 2)*FZoom div 100;
      if p.x<x then p.x:=x
      else if p.x+x>screen.width then p.x:=screen.width-x;
      if p.y<y then p.y:=y
      else if p.y+y>screen.height then p.y:=screen.height-y;
      t.Left:=p.x-x;
      t.Top:=p.y-y;
      t.Right:=p.x+x;
      t.bottom:=p.y+y;

      //Draw the area around the mouse
      FZCanvas.CopyRect(rect(0,0,self.width,self.height),Fcanvas,t);
   end;
end;
{**************************************************}
procedure TJvZoom.SetActive(const Value: boolean);
begin
   FActive := Value;
   FTimer.Enabled:=Value;
end;
{**************************************************}
procedure TJvZoom.SetDelay(const Value: cardinal);
begin
   FDelay := Value;
   Ftimer.Interval:=Value;
end;
{**************************************************}
procedure TJvZoom.WMSize(var Message: TWMSize);
begin
   //On resize, refresh it
   inherited;
   PaintMe(self);
end;
{**************************************************}
end.
